----------
-- Name: WOOGLED
-- Description: A player and pet cooldown-tracking addon for WoW
-- Author: Woogs - Greymane (US)
-- Version: 2.x.x-alpha
-- License: All Rights Reserved
----------


local WOOGLED = WOOGLED;
local Zones = WOOGLED.Zones;
local Libs = WOOGLED.Libs;
local Spells = WOOGLED.Spells;
local Tools = WOOGLED.Tools;
local Units = WOOGLED.Units;
local CooldownFrames = WOOGLED.CooldownFrames;
local OptionsFrame = WOOGLED.OptionsFrame;
local MinimapFrame = WOOGLED.MinimapFrame;


CooldownFrames.frames = {};

CooldownFrames.guidIndexByGroupName = {};
CooldownFrames.groupNameIndexByGuid = {};

CooldownFrames.petLinkGuidIndexByGroupName = {};
CooldownFrames.groupNameIndexByPetLinkGuid = {};

CooldownFrames.groupsTableIndexByGroupName = {};

local toggles = WOOGLED.toggles;
local cf = CooldownFrames.frames;
local extraFrameCounter = 0;
local avaliableExtraFrameArray = {};
local unavaliableExtraFrameCounter = 0;
local unavaliableExtraFrameIndex = {};
local groupsTableIndexByGroupName = CooldownFrames.groupsTableIndexByGroupName;

local testPet = {
	guid = 1,
	name = 'Test',
	server = 'Pet',
	cooldowns = {
		tables = {
			pet_test_base_bar1 = Spells.pet.cooldown.class.test.base.bar1;
			pet_test_base_bar2 = Spells.pet.cooldown.class.test.base.bar2;
			pet_test_base_ignoreInterval = Spells.pet.cooldown.class.test.base.ignoreInterval;
		},
		maxDurations = {
			pet_test_base_bar1 = Spells.pet.cooldown.class.test.base.bar1.maxDuration or Spells.pet.cooldown.class.test.base.bar1.baseDuration;
			pet_test_base_bar2 = Spells.pet.cooldown.class.test.base.bar2.maxDuration or Spells.pet.cooldown.class.test.base.bar2.baseDuration;
			pet_test_base_ignoreInterval = Spells.pet.cooldown.class.test.base.ignoreInterval.maxDuration or Spells.pet.cooldown.class.test.base.ignoreInterval.baseDuration;
		},
		timestamps = {
			pet_test_base_bar1 = 0;
			pet_test_base_bar2 = 0;
			pet_test_base_ignoreInterval = 0;
		},
	},
};

local testUnit = {
	guid = 1,
	name = 'Test',
	unitType = 'player',
	server = 'Player',
	class = 'druid',
	spec = 'feral',
	petLinkTable = testPet;
	cooldowns = {
		tables = {
			player_test_base_bar1 = Spells.player.cooldown.all.test.base.bar1;
			player_test_base_bar2 = Spells.player.cooldown.all.test.base.bar2;
			player_test_base_ignoreInterval = Spells.player.cooldown.all.test.base.ignoreInterval;
		},
		maxDurations = {
			player_test_base_bar1 = Spells.player.cooldown.all.test.base.bar1.maxDuration or Spells.player.cooldown.all.test.base.bar1.baseDuration;
			player_test_base_bar2 = Spells.player.cooldown.all.test.base.bar2.maxDuration or Spells.player.cooldown.all.test.base.bar2.baseDuration;
			player_test_base_ignoreInterval = Spells.player.cooldown.all.test.base.ignoreInterval.maxDuration or Spells.player.cooldown.all.test.base.ignoreInterval.baseDuration;
		},
		timestamps = {
			player_test_base_bar1 = 0;
			player_test_base_bar2 = 0;
			player_test_base_ignoreInterval = 0;
		},
	},
};


function CooldownFrames.build()
	CooldownFrames.buildCooldownFrame('tf', 'tf', true, false);
	cf.tf.pinned = false;
	CooldownFrames.buildCooldownFrame('ff', 'ff', false, false);
	
	extraFrameCounter = extraFrameCounter + 1;
	local groupName = 'ef'..extraFrameCounter;
	CooldownFrames.buildCooldownFrame(groupName, 'ef', false, true);
	CooldownFrames.setGroupUnit(groupName, pUnitId);
	unavaliableExtraFrameIndex[groupName] = 1;
	unavaliableExtraFrameCounter = unavaliableExtraFrameCounter + 1;
	CooldownFrames.closeExtraFrame(groupName);
end


function CooldownFrames.buildCooldownFrame(pGroupName, pOptionsTableName, pBuildPinButton, pBuildCloseButton)
	
	-- Setup data structure
	cf[pGroupName] = {
		options = WOOGLED_OPTIONS[pOptionsTableName],
		optionsName = pOptionsTableName,
		unit = {
			-- guid
			-- name
			-- server
			-- type
			-- table
		},
		tables = {
			cooldownNameArray = {},
			unitTableIndex = {}, -- Indexed by cooldown name
			cooldownStateIndex = {}, -- Indexed by cooldown name
			expirationTimestampIndex = {}, -- Indexed by cooldown name
			midpointTimestampIndex = {}, -- Indexed by cooldown name
		},
		frames = {
			header = {
				class = {
					frameArray = {},
					-- currentFrame
				},
				spec = {
					frameArray = {},
					-- currentFrame
				},
				pet = {
					frameArray = {},
					-- currentFrame
				},
			},
			rows = {},
		}
	};
	if ( groupsTableIndexByGroupName[pOptionsTableName] == nil ) then groupsTableIndexByGroupName[pOptionsTableName] = {}; end
	table.insert(groupsTableIndexByGroupName[pOptionsTableName], pGroupName);
	
	-- Build header
	
	-- Declare local variables
	local f = cf[pGroupName].frames.header;
	local globalPrefix = 'WOOGLED_COOLDOWNFRAMES_FRAMES_'..strupper(pGroupName)..'_FRAMES_HEADER_';
	
	f.anchor = CreateFrame('Frame', globalPrefix..'ANCHOR', UIParent);
	f.anchor:SetPoint('CENTER');
	f.anchor:SetClampedToScreen(true);
	f.anchor:EnableMouse(true);
	f.anchor:SetMovable(true);
	f.anchor:SetScript('OnDragStart', function() f.anchor:StartMoving(); f.anchor.isMoving = true; end);
	f.anchor:SetScript('OnDragStop', function() f.anchor:StopMovingOrSizing(); f.anchor.isMoving = false; end);
	f.anchor:Hide(); -- starts out hidden; options will show if needed
	f.anchor.texture2 = f.anchor:CreateTexture();
	f.anchor.texture2:SetAllPoints(f.anchor);
	Tools.setupTexture(f.anchor);
	
	for className, iconPath in pairs ( Libs.icons.player.class ) do
		f.class[className] = CreateFrame('Frame', globalPrefix..'CLASS_'..strupper(className), f.anchor);
		f.class[className]:Hide();
		Tools.setupTexture(f.class[className], iconPath);
		table.insert(f.class.frameArray, className);
		
		f.pet[className] = CreateFrame('Button', globalPrefix..'PET_'..strupper(className), f.anchor);
		f.pet[className]:Hide();
		Tools.setupTexture(f.pet[className], iconPath);
		table.insert(f.pet.frameArray, className);
	end
	
	for className, iconPath in pairs ( Libs.icons.pet.class ) do
		f.class[className] = CreateFrame('Frame', globalPrefix..'CLASS_'..strupper(className), f.anchor);
		f.class[className]:Hide();
		Tools.setupTexture(f.class[className], iconPath);
		table.insert(f.class.frameArray, className);
		
		f.pet[className] = CreateFrame('Button', globalPrefix..'PET_'..strupper(className), f.anchor);
		f.pet[className]:Hide();
		Tools.setupTexture(f.pet[className], iconPath);
		table.insert(f.pet.frameArray, className);
	end
	
	for className, classTable in pairs ( Libs.icons.player.spec ) do
		for specName, iconPath in pairs ( classTable ) do
			local frameName = className..'_'..specName;
			f.spec[frameName] = CreateFrame('Frame', globalPrefix..'SPEC_'..strupper(className), f.anchor);
			f.spec[frameName]:Hide();
			Tools.setupTexture(f.spec[frameName], iconPath);
			table.insert(f.spec.frameArray, frameName);
		end
	end
	
	f.pet.generic = CreateFrame('Button', globalPrefix..'PET_GENERIC', f.anchor);
	f.pet.generic:Hide();
	Tools.setupTexture(f.pet.generic, Libs.icons.misc.petLink);
	table.insert(f.pet.frameArray, 'generic');
	
	if ( pBuildPinButton ) then
		f.pin = CreateFrame('Button', globalPrefix..'PIN', f.anchor);
		f.pin:SetPoint('TOPRIGHT');
		f.pin:SetScript('OnClick', function() CooldownFrames.setPin(true, pGroupName); end);
		Tools.setupTexture(f.pin, Libs.icons.misc.unpin);
		
		f.unpin = CreateFrame('Button', globalPrefix..'UNPIN', f.anchor);
		f.unpin:SetPoint('TOPRIGHT');
		f.unpin:SetScript('OnClick', function() CooldownFrames.setPin(false, pGroupName); end);
		f.unpin:Hide();
		Tools.setupTexture(f.unpin, Libs.icons.misc.pin);
	end
	
	if ( pBuildCloseButton ) then
		f.close = CreateFrame('Button', globalPrefix..'CLOSE', f.anchor);
		f.close:SetPoint('TOPRIGHT');
		f.close:SetScript('OnClick', function() CooldownFrames.closeExtraFrame(pGroupName); end);
		Tools.setupTexture(f.close, Libs.icons.misc.close);
	end
	
	f.name = f.anchor:CreateFontString('FontString', 'ARTWORK', 'GameFontNormal');
		
	f.server = f.anchor:CreateFontString('FontString', 'ARTWORK', 'GameFontNormal');
	f.server:SetPoint('LEFT', f.name, 'RIGHT');
	
	-- Build rows
	
	-- Declare local variables
	local f = cf[pGroupName].frames.rows;
	
	for cooldownName, cooldownTable in pairs ( Spells.cooldownIndex ) do
		
		local globalPrefix = 'WOOGLED_COOLDOWNFRAMES_FRAMES_'..strupper(pGroupName)..'FRAMES_ROWS_'..strupper(cooldownName)..'_';
		
		f[cooldownName..'_background'] = CreateFrame('Frame', globalPrefix..'BACKGROUND', cf[pGroupName].frames.header.anchor);
		f[cooldownName..'_background']:SetPoint('CENTER');
		f[cooldownName..'_background']:Hide();
		Tools.setupTexture(f[cooldownName..'_background']);
	
		f[cooldownName..'_icon'] = CreateFrame('Frame', globalPrefix..'ICON', f[cooldownName..'_background']);
		f[cooldownName..'_icon']:SetPoint('LEFT');
		Tools.setupTexture(f[cooldownName..'_icon'], cooldownTable.icon or Tools.getIconPath(cooldownTable.mainId));

		f[cooldownName..'_bar1'] = CreateFrame('StatusBar', globalPrefix..'BAR1', f[cooldownName..'_background']);
		f[cooldownName..'_bar1']:SetPoint('LEFT', f[cooldownName..'_icon'], 'RIGHT');
			
		if ( not cooldownTable.ignoreInterval ) then
			f[cooldownName..'_divider'] = CreateFrame('Frame', globalPrefix..'_DIVIDER', f[cooldownName..'_bar1']);
			f[cooldownName..'_divider']:SetPoint('CENTER', f[cooldownName..'_bar1'], 'RIGHT');
			Tools.setupTexture(f[cooldownName..'_divider']);
		
			f[cooldownName..'_bar2'] = CreateFrame('StatusBar', globalPrefix..'BAR2', f[cooldownName..'_background']);
			f[cooldownName..'_bar2']:SetPoint('LEFT', f[cooldownName..'_bar1'], 'RIGHT');
		end
		
		f[cooldownName..'_textStep1'] = CreateFrame('Frame', globalPrefix..'TEXTSTEP1', f[cooldownName..'_background']);

		f[cooldownName..'_textStep2'] = CreateFrame('Frame', globalPrefix..'TEXTSTEP2', f[cooldownName..'_textStep1']);

		f[cooldownName..'_name'] = f[cooldownName..'_textStep2']:CreateFontString('FontString', 'OVERLAY ', 'GameFontNormal');
		f[cooldownName..'_name']:SetJustifyH('LEFT');
		f[cooldownName..'_name']:SetText(cooldownTable.name or Tools.getSpellName(cooldownTable.mainId));

		f[cooldownName..'_cooldown'] = f[cooldownName..'_textStep2']:CreateFontString('FontString', 'OVERLAY ', 'GameFontNormal');	
		f[cooldownName..'_cooldown']:SetPoint('RIGHT', f[cooldownName..'_background'], 'RIGHT', WOOGLED_OPTIONS[pOptionsTableName].row.textIndent*-1, 0);
	end
	
	CooldownFrames.setAllOptions(pGroupName);
end


function CooldownFrames.setAllOptions(pGroupName)

	local optionsName = cf[pGroupName].optionsName

	-- Endpoints
	CooldownFrames.endpoint_setEndPoint1(nil, optionsName);
	CooldownFrames.endpoint_setEndPoint2(nil, optionsName);
	
	-- General anchor
	CooldownFrames.setLock(nil, optionsName);
	CooldownFrames.setScale(nil, optionsName);
	CooldownFrames.anchor_setWidth(nil, optionsName);
	CooldownFrames.anchor_setHeight(nil, optionsName);

	-- Anchor background
	CooldownFrames.anchor_setBackgroundColor(nil, nil, nil, nil, optionsName);
	CooldownFrames.anchor_setBackgroundTextureVisibility(nil, optionsName);
	CooldownFrames.anchor_setBackgroundTexture(nil, optionsName);
	
	-- Class icon
	CooldownFrames.anchor_setClassVisibility(nil, optionsName);
	CooldownFrames.anchor_setClassLength(nil, optionsName);
	CooldownFrames.anchor_setClassAlpha(nil, optionsName);
	
	-- Spec icon
	CooldownFrames.anchor_setSpecVisibility(nil, optionsName);
	CooldownFrames.anchor_setSpecLength(nil, optionsName);
	CooldownFrames.anchor_setSpecAlpha(nil, optionsName);
	
	-- Pet link icon
	CooldownFrames.anchor_setPetVisibility(nil, optionsName);
	CooldownFrames.anchor_setPetLength(nil, optionsName);
	CooldownFrames.anchor_setPetAlpha(nil, optionsName);
	
	-- Pin button
	if ( cf[pGroupName].frames.header.pin ~= nil ) then
		CooldownFrames.anchor_setPinLength(nil, optionsName);
		CooldownFrames.anchor_setPinAlpha(nil, optionsName);
		CooldownFrames.anchor_setPinVisibility(nil, optionsName);
	end
	
	-- Close button
	if ( cf[pGroupName].frames.header.close ~= nil ) then
		CooldownFrames.anchor_setCloseLength(nil, optionsName);
		CooldownFrames.anchor_setCloseAlpha(nil, optionsName);
	end

	-- Unit name
	CooldownFrames.anchor_setNameVisibility(nil, optionsName);
	CooldownFrames.anchor_setNameColor(nil, nil, nil, nil, optionsName);
	CooldownFrames.anchor_setNameFont(nil, optionsName);
	CooldownFrames.anchor_setNameFontSize(nil, optionsName);
	
	-- Unit server
	CooldownFrames.anchor_setServerVisibility(nil, optionsName);
	CooldownFrames.anchor_setServerColor(nil, nil, nil, nil, optionsName);
	CooldownFrames.anchor_setServerFont(nil, optionsName);
	CooldownFrames.anchor_setServerFontSize(nil, optionsName);

	-- General row
	CooldownFrames.row_setSpacing(nil, optionsName);
	CooldownFrames.row_setAlignment(nil, optionsName);
	CooldownFrames.row_setBuildDirection(nil, optionsName);
	CooldownFrames.row_setStackDirection(nil, optionsName);
	CooldownFrames.row_setSortType(nil, optionsName);

	-- Unit specific rows
	local unitTypes = {'player', 'pet'};
	
	for i, unitType in pairs ( unitTypes ) do
		
		-- General
		CooldownFrames.row_setWidth(nil, optionsName, unitType);
		CooldownFrames.row_setHeight(nil, optionsName, unitType);
		
		-- Background
		CooldownFrames.row_setBackgroundTextureVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setBackgroundTexture(nil, optionsName, unitType);
		CooldownFrames.row_setBackgroundColor(nil, nil, nil, nil, optionsName, unitType);
		
		-- Cooldown icon
		CooldownFrames.row_setIconVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setIconLength(nil, optionsName, unitType);
		CooldownFrames.row_setIconAlpha(nil, optionsName, unitType);
		
		-- Cooldown name
		CooldownFrames.row_setNameVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setNameFont(nil, optionsName, unitType);
		CooldownFrames.row_setNameFontSize(nil, optionsName, unitType);
		CooldownFrames.row_setNameColor(nil, nil, nil, nil, optionsName, unitType);
		CooldownFrames.row_setNameHighlightVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setNameHighlightColor(nil, nil, nil, nil, optionsName, unitType);
		
		-- Cooldown duration
		CooldownFrames.row_setCooldownVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setCooldownFont(nil, optionsName, unitType);
		CooldownFrames.row_setCooldownFontSize(nil, optionsName, unitType);
		CooldownFrames.row_setCooldownColor(nil, nil, nil, nil, optionsName, unitType);
		CooldownFrames.row_setCooldownHighlightColorVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setCooldownHighlightColor(nil, nil, nil, nil, optionsName, unitType);
		CooldownFrames.row_setCooldownEnableCutoff(nil, optionsName, unitType);
		CooldownFrames.row_setCooldownCutoff(nil, optionsName, unitType);
		CooldownFrames.row_setCooldownShowInMinutes(nil, optionsName, unitType);		
		CooldownFrames.row_setCooldownCountdownBothBars(nil, optionsName, unitType);
		
		-- Bars
		CooldownFrames.row_setBarsVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setBarsMode(nil, optionsName, unitType);
		
		-- Bar 1
		CooldownFrames.row_setBar1Color(nil, nil, nil, nil, optionsName, unitType);
		CooldownFrames.row_setBar1TextureVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setBar1Texture(nil, optionsName, unitType);
		
		-- Bar 2
		CooldownFrames.row_setBar2Color(nil, nil, nil, nil, optionsName, unitType);
		CooldownFrames.row_setBar2TextureVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setBar2Texture(nil, optionsName, unitType);
		
		-- Divider
		CooldownFrames.row_setDividerVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setDividerWidth(nil, optionsName, unitType);
		CooldownFrames.row_setDividerColor(nil, nil, nil, nil, optionsName, unitType);
		CooldownFrames.row_setDividerTextureVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setDividerTexture(nil, optionsName, unitType);
		
		-- Ignore interval
		CooldownFrames.row_setIgnoreIntervalEnable(nil, optionsName, unitType);
		CooldownFrames.row_setIgnoreIntervalColor(nil, nil, nil, nil, optionsName, unitType);
		CooldownFrames.row_setIgnoreIntervalTextureVisibility(nil, optionsName, unitType);
		CooldownFrames.row_setIgnoreIntervalTexture(nil, optionsName, unitType);
	end
end


function CooldownFrames.update()
	if ( cf.tf.frames.header.anchor:IsShown() ) then
		CooldownFrames.updateFrames(cf.tf);
	end
	if ( cf.ff.frames.header.anchor:IsShown() ) then
		CooldownFrames.updateFrames(cf.ff);
	end
	if ( unavaliableExtraFrameCounter > 0 ) then
		for groupName, _ in pairs ( unavaliableExtraFrameIndex ) do
			if ( cf[groupName].frames.header.anchor:IsShown() ) then
				CooldownFrames.updateFrames(cf[groupName]);
			end
		end
	end
end


function CooldownFrames.updateFrames(pGroupTable)
	if ( #pGroupTable.tables.cooldownNameArray > 0 ) then
		for i, cooldownName in pairs ( pGroupTable.tables.cooldownNameArray ) do
		
			local unitTable = pGroupTable.tables.unitTableIndex[cooldownName];
			local cooldownTable = unitTable.cooldowns.tables[cooldownName];
			local unitType = Spells.unitTypeIndexByCooldownName[cooldownName];
			
			local updateTime = WOOGLED.time.now;
			if ( pGroupTable.options.row.bars[unitType].mode == 'drain' ) then
				updateTime = updateTime * -1;
			end
			
			if ( pGroupTable.tables.cooldownStateIndex[cooldownName] == 'primary' ) then
				pGroupTable.frames.rows[cooldownName..'_bar1']:SetValue(updateTime);
				if ( pGroupTable.tables.midpointTimestampIndex[cooldownName] ~= nil and WOOGLED.time.now > pGroupTable.tables.midpointTimestampIndex[cooldownName] ) then				
					pGroupTable.frames.rows[cooldownName..'_bar2']:SetValue(updateTime);
					pGroupTable.tables.cooldownStateIndex[cooldownName] = 'secondary';
					CooldownFrames.updateRowNameColor(pGroupTable, cooldownName);
					CooldownFrames.updateRowCountdownColor(pGroupTable, cooldownName);
				end
			else
				pGroupTable.frames.rows[cooldownName..'_bar2']:SetValue(updateTime);
			end
			
			CooldownFrames.updateCooldownBarText(pGroupTable, cooldownName);
			
			if ( WOOGLED.time.now >= pGroupTable.tables.expirationTimestampIndex[cooldownName] ) then
				CooldownFrames.onCooldownFinish(pGroupTable, cooldownName, i);
			end
		end
	end
end


function CooldownFrames.updateGroupTables(pGroupTable)
	
	-- Hide displayed rows and clear display tables
	CooldownFrames.clearGroupTables(pGroupTable);
	
	-- Get unit cooldown data
	if ( pGroupTable.unit.table ~= nil ) then
	
		CooldownFrames.getCooldownData(pGroupTable, pGroupTable.unit.table);
	
		-- Get unit's pet's cooldown data
		if ( pGroupTable.options.showPet and pGroupTable.unit.table.unitType == 'player' and pGroupTable.unit.table.petLinkTable ~= nil ) then
			CooldownFrames.getCooldownData(pGroupTable, pGroupTable.unit.table.petLinkTable);
		end
	end
	
	if ( #pGroupTable.tables.cooldownNameArray > 0 ) then
		CooldownFrames.updateRowPositions(pGroupTable);
	end
end


function CooldownFrames.clearGroupTables(pGroupTable)
	
	for i, cooldownName in pairs ( pGroupTable.tables.cooldownNameArray ) do
		pGroupTable.frames.rows[cooldownName..'_background']:Hide();
		pGroupTable.tables.cooldownNameArray[i] = nil;
	end
	
	pGroupTable.tables.unitTableIndex = {};
end


function CooldownFrames.updateUnitIcons(pGroupTable)
	CooldownFrames.updateClassIcon(pGroupTable);
	CooldownFrames.updateSpecIcon(pGroupTable);
	CooldownFrames.updatePetIcon(pGroupTable);
end


function CooldownFrames.clear()
	CooldownFrames.clearGroupTables(cf.tf);
	CooldownFrames.updateUnitIcons(cf.tf);
	
	CooldownFrames.clearGroupTables(cf.ff);
	CooldownFrames.updateUnitIcons(cf.ff);
end


function CooldownFrames.getCooldownData(pGroupTable, pUnitTable)
	for cooldownName, timestamp in pairs ( pUnitTable.cooldowns.timestamps ) do
		if ( WOOGLED_OPTIONS.ignoreSpells[cooldownName] == nil ) then
			local expirationTimestamp, midpointTimestamp = CooldownFrames.getCooldownExpirationTimestamp(pGroupTable, pUnitTable, cooldownName);
			if ( timestamp ~= 0 and WOOGLED.time.now < expirationTimestamp ) then -- 0 == a cooldown that has been reset
				CooldownFrames.addCooldownToDisplay(pGroupTable, pUnitTable, cooldownName, expirationTimestamp, midpointTimestamp)
			end
		end
	end
end


function CooldownFrames.addCooldownToDisplay(pGroupTable, pUnitTable, pCooldownName, pExpirationTimestamp, pMidpointTimestamp)
	
	if ( pGroupTable.tables.unitTableIndex[pCooldownName] == nil ) then
		table.insert(pGroupTable.tables.cooldownNameArray, pCooldownName);
		pGroupTable.tables.unitTableIndex[pCooldownName] = pUnitTable;
	end
	
	if ( pMidpointTimestamp ~= nil and WOOGLED.time.now > pMidpointTimestamp ) then
		pGroupTable.tables.cooldownStateIndex[pCooldownName] = 'secondary';
	else
		pGroupTable.tables.cooldownStateIndex[pCooldownName] = 'primary';
	end
	
	pGroupTable.tables.midpointTimestampIndex[pCooldownName] = pMidpointTimestamp;
	pGroupTable.tables.expirationTimestampIndex[pCooldownName] = pExpirationTimestamp;
	
	CooldownFrames.updateRowNameColor(pGroupTable, pCooldownName); -- Just pass frame reference?
	CooldownFrames.updateRowCountdownColor(pGroupTable, pCooldownName); -- Just pass frame reference?
	CooldownFrames.updateRowBackgroundColor(pGroupTable, pCooldownName); -- Just pass frame reference?
	CooldownFrames.updatePrimaryBarColor(pGroupTable, pCooldownName); -- Just pass frame reference?
	CooldownFrames.updateSecondaryBarColor(pGroupTable, pCooldownName); -- Just pass frame reference?
	CooldownFrames.updateDividerColor(pGroupTable, pCooldownName); -- Just pass frame reference?
	CooldownFrames.updateCooldownBarText(pGroupTable, pCooldownName); -- Just pass frame reference?
	CooldownFrames.updateStatusBarPositions(pGroupTable, pCooldownName); -- Just pass frame reference?

	pGroupTable.frames.rows[pCooldownName..'_background']:Show();
end


function CooldownFrames.updateRowNameColor(pGroupTable, pCooldownName)
	local unitType, durationLeft = Spells.unitTypeIndexByCooldownName[pCooldownName];
	if ( Spells.cooldownIndex[pCooldownName].ignoreInterval and pGroupTable.options.row.ignoreInterval[unitType].enable ) then
		Tools.setTextColor(pGroupTable.frames.rows[pCooldownName..'_name'], pGroupTable.options.row.name[unitType].ignoreIntervalColor);
	elseif ( pGroupTable.tables.midpointTimestampIndex[pCooldownName] ~= nil and WOOGLED.time.now > pGroupTable.tables.midpointTimestampIndex[pCooldownName] and pGroupTable.options.row.name[unitType].showHighlight ) then	
		Tools.setTextColor(pGroupTable.frames.rows[pCooldownName..'_name'], pGroupTable.options.row.name[unitType].highlightColor);
	else
		Tools.setTextColor(pGroupTable.frames.rows[pCooldownName..'_name'], pGroupTable.options.row.name[unitType].color);
	end
end


function CooldownFrames.updateRowCountdownColor(pGroupTable, pCooldownName)
	local unitType, durationLeft = Spells.unitTypeIndexByCooldownName[pCooldownName];
	if ( Spells.cooldownIndex[pCooldownName].ignoreInterval and pGroupTable.options.row.ignoreInterval[unitType].enable ) then
		Tools.setTextColor(pGroupTable.frames.rows[pCooldownName..'_cooldown'], pGroupTable.options.row.cooldown[unitType].ignoreIntervalColor);
	elseif ( pGroupTable.tables.midpointTimestampIndex[pCooldownName] ~= nil and WOOGLED.time.now > pGroupTable.tables.midpointTimestampIndex[pCooldownName] and pGroupTable.options.row.cooldown[unitType].showHighlight ) then	
		Tools.setTextColor(pGroupTable.frames.rows[pCooldownName..'_cooldown'], pGroupTable.options.row.cooldown[unitType].highlightColor);
	else
		Tools.setTextColor(pGroupTable.frames.rows[pCooldownName..'_cooldown'], pGroupTable.options.row.cooldown[unitType].color);
	end
end


function CooldownFrames.updateCooldownBarText(pGroupTable, pCooldownName)
	local unitType, durationLeft = Spells.unitTypeIndexByCooldownName[pCooldownName];
	if ( pGroupTable.options.row.cooldown[unitType].countdownBothBars and pGroupTable.tables.midpointTimestampIndex[pCooldownName] ~= nil and WOOGLED.time.now < pGroupTable.tables.midpointTimestampIndex[pCooldownName] ) then
		durationLeft = pGroupTable.tables.midpointTimestampIndex[pCooldownName] - WOOGLED.time.now;
	else
		durationLeft = pGroupTable.tables.expirationTimestampIndex[pCooldownName] - WOOGLED.time.now;
	end
	if ( pGroupTable.options.row.cooldown[unitType].show and (not pGroupTable.options.row.cooldown[unitType].enableCutoff or (pGroupTable.options.row.cooldown[unitType].enableCutoff and durationLeft < pGroupTable.options.row.cooldown[unitType].cutoffThreshold)) ) then
		if ( pGroupTable.options.row.cooldown[unitType].showInMinutes ) then
			if ( durationLeft >= 60000 ) then
				pGroupTable.frames.rows[pCooldownName..'_cooldown']:SetText(Tools.roundNumber(durationLeft / 60000)..'m');
			elseif ( durationLeft >= 1000 ) then
				pGroupTable.frames.rows[pCooldownName..'_cooldown']:SetText(Tools.roundNumber(durationLeft / 1000));
			else
				pGroupTable.frames.rows[pCooldownName..'_cooldown']:SetText(Tools.roundNumber(durationLeft / 100) / 10);
			end
		else
			if ( durationLeft >= 1000 ) then
				pGroupTable.frames.rows[pCooldownName..'_cooldown']:SetText(Tools.roundNumber(durationLeft / 1000));
			else
				pGroupTable.frames.rows[pCooldownName..'_cooldown']:SetText(Tools.roundNumber(durationLeft / 100) / 10);
			end
		end
		pGroupTable.frames.rows[pCooldownName..'_name']:SetWidth(CooldownFrames.getBarWidth(pGroupTable, unitType) - pGroupTable.frames.rows[pCooldownName..'_cooldown']:GetWidth() - (pGroupTable.options.row.textIndent * 3));
		pGroupTable.frames.rows[pCooldownName..'_cooldown']:Show();
	else
		pGroupTable.frames.rows[pCooldownName..'_name']:SetWidth(CooldownFrames.getBarWidth(pGroupTable, unitType) - (pGroupTable.options.row.textIndent * 3));
		pGroupTable.frames.rows[pCooldownName..'_cooldown']:Hide();
	end
	if ( pGroupTable.options.row.name[unitType].show ) then
		pGroupTable.frames.rows[pCooldownName..'_name']:Show();
	else
		pGroupTable.frames.rows[pCooldownName..'_name']:Hide();
	end
end


function CooldownFrames.updateRowPositions(pGroupTable)
	CooldownFrames.sortCooldownFrameIndex(pGroupTable);
	local anchorPoint, referencePoint, verticalSpacing;
	if ( pGroupTable.options.row.buildDirection == 'up' ) then
		if ( pGroupTable.options.row.alignment == 'left' ) then
			anchorPoint = 'BOTTOMLEFT';
			referencePoint = 'TOPLEFT';
		elseif ( pGroupTable.options.row.alignment == 'right' ) then
			anchorPoint = 'BOTTOMRIGHT';
			referencePoint = 'TOPRIGHT';
		else
			anchorPoint = 'BOTTOM';
			referencePoint = 'TOP';
		end
		verticalSpacing = pGroupTable.options.row.background.spacing;
	else
		if ( pGroupTable.options.row.alignment == 'left' ) then
			anchorPoint = 'TOPLEFT';
			referencePoint = 'BOTTOMLEFT';
		elseif ( pGroupTable.options.row.alignment == 'right' ) then
			anchorPoint = 'TOPRIGHT';
			referencePoint = 'BOTTOMRIGHT';
		else
			anchorPoint = 'TOP';
			referencePoint = 'BOTTOM';
		end
		verticalSpacing = pGroupTable.options.row.background.spacing * -1;
	end
	for i, cooldownName in pairs ( pGroupTable.tables.cooldownNameArray ) do
		pGroupTable.frames.rows[cooldownName..'_background']:ClearAllPoints();
		if ( i == 1 ) then
			pGroupTable.frames.rows[cooldownName..'_background']:SetPoint(anchorPoint, pGroupTable.frames.header.anchor, referencePoint, 0, verticalSpacing);
		else
			pGroupTable.frames.rows[cooldownName..'_background']:SetPoint(anchorPoint, pGroupTable.frames.rows[pGroupTable.tables.cooldownNameArray[i-1]..'_background'], referencePoint, 0, verticalSpacing);
		end
	end
end


function CooldownFrames.sortCooldownFrameIndex(pGroupTable)
	table.sort(pGroupTable.tables.cooldownNameArray, function(a, b) 
		if ( a == nil ) then return false; end
		if ( b == nil ) then return true; end
		local unitTableA = pGroupTable.tables.unitTableIndex[a];
		local unitTableB = pGroupTable.tables.unitTableIndex[b];
		if ( pGroupTable.options.row.sortBy == 'event' ) then
			if ( pGroupTable.options.row.stackDirection == 'down' ) then
				return unitTableA.cooldowns.timestamps[a] < unitTableB.cooldowns.timestamps[b];
			else
				return unitTableA.cooldowns.timestamps[a] > unitTableB.cooldowns.timestamps[b];
			end
		else
			if ( pGroupTable.options.row.stackDirection == 'down' ) then
				return unitTableA.cooldowns.timestamps[a] + unitTableA.cooldowns.maxDurations[a] < unitTableB.cooldowns.timestamps[b] + unitTableB.cooldowns.maxDurations[b];
			else
				return unitTableA.cooldowns.timestamps[a] + unitTableA.cooldowns.maxDurations[a] > unitTableB.cooldowns.timestamps[b] + unitTableB.cooldowns.maxDurations[b];
			end
		end
	end);
end


function CooldownFrames.updateStatusBarPositions(pGroupTable, pCooldownName)
	local unitType = Spells.unitTypeIndexByCooldownName[pCooldownName];
	if ( pGroupTable.options.row.bars[unitType].show ) then
		local cooldownTable, barWidth, unitType = pGroupTable.tables.unitTableIndex[pCooldownName].cooldowns.tables[pCooldownName], CooldownFrames.getBarWidth(pGroupTable, unitType), Spells.unitTypeIndexByCooldownName[pCooldownName];
		local startTimestamp, midpointTimestamp, expirationTimestamp = pGroupTable.tables.unitTableIndex[pCooldownName].cooldowns.timestamps[pCooldownName], pGroupTable.tables.midpointTimestampIndex[pCooldownName], pGroupTable.tables.expirationTimestampIndex[pCooldownName];
		
		local leftTimestamp, middleTimestamp, rightTimestamp, updateTime;
		if ( pGroupTable.options.row.bars[unitType].mode == 'drain' ) then
			leftTimestamp, rightTimestamp = expirationTimestamp*-1, startTimestamp*-1;
			if ( midpointTimestamp ~= nil ) then
				middleTimestamp = midpointTimestamp*-1;
			end
			updateTime = WOOGLED.time.now * -1;
		else
			-- Defaults to fill
			leftTimestamp, rightTimestamp = startTimestamp, expirationTimestamp;
			if ( midpointTimestamp ~= nil ) then
				middleTimestamp = midpointTimestamp;
			end
			updateTime = WOOGLED.time.now;
		end
		
		if ( cooldownTable.ignoreInterval ) then
			pGroupTable.frames.rows[pCooldownName..'_bar1']:SetWidth(barWidth);
			pGroupTable.frames.rows[pCooldownName..'_bar1']:SetMinMaxValues(leftTimestamp, rightTimestamp);
			pGroupTable.frames.rows[pCooldownName..'_bar1']:SetValue(updateTime);
			pGroupTable.frames.rows[pCooldownName..'_bar1']:Show();
		else
			if ( midpointTimestamp ~= nil ) then
				local primaryDuration, secondaryDuration, fullDuration = midpointTimestamp-startTimestamp, expirationTimestamp-midpointTimestamp, expirationTimestamp-startTimestamp;
				
				if ( pGroupTable.options.row.bars[unitType].mode == 'drain' ) then
					pGroupTable.frames.rows[pCooldownName..'_bar1']:SetMinMaxValues(middleTimestamp, rightTimestamp);
					pGroupTable.frames.rows[pCooldownName..'_bar2']:SetMinMaxValues(leftTimestamp, middleTimestamp);
				else
					-- Defaults to fill
					pGroupTable.frames.rows[pCooldownName..'_bar1']:SetMinMaxValues(leftTimestamp, middleTimestamp);
					pGroupTable.frames.rows[pCooldownName..'_bar2']:SetMinMaxValues(middleTimestamp, rightTimestamp);
				end
				
				pGroupTable.frames.rows[pCooldownName..'_bar1']:SetWidth((primaryDuration/fullDuration)*barWidth);
				pGroupTable.frames.rows[pCooldownName..'_bar1']:SetValue(updateTime);
				pGroupTable.frames.rows[pCooldownName..'_bar1']:Show();
				
				pGroupTable.frames.rows[pCooldownName..'_bar2']:SetWidth((secondaryDuration/fullDuration)*barWidth);
				pGroupTable.frames.rows[pCooldownName..'_bar2']:SetValue(updateTime);
				pGroupTable.frames.rows[pCooldownName..'_bar2']:Show();
				if ( pGroupTable.options.row.divider[unitType].show ) then
					pGroupTable.frames.rows[pCooldownName..'_divider']:SetWidth(pGroupTable.options.row.divider[unitType].width);
					pGroupTable.frames.rows[pCooldownName..'_divider']:Show();
				else
					pGroupTable.frames.rows[pCooldownName..'_divider']:Hide();
				end
			else
				pGroupTable.frames.rows[pCooldownName..'_bar1']:SetWidth(barWidth);
				pGroupTable.frames.rows[pCooldownName..'_bar1']:SetMinMaxValues(leftTimestamp, rightTimestamp);
				pGroupTable.frames.rows[pCooldownName..'_bar1']:SetValue(updateTime);
				pGroupTable.frames.rows[pCooldownName..'_bar1']:Show();
				pGroupTable.frames.rows[pCooldownName..'_bar2']:Hide();
				pGroupTable.frames.rows[pCooldownName..'_divider']:Hide();
			end
		end
	else
		pGroupTable.frames.rows[pCooldownName..'_bar1']:Hide();
		if ( pGroupTable.frames.rows[pCooldownName..'_bar2'] ~= nil ) then
			pGroupTable.frames.rows[pCooldownName..'_bar2']:Hide();
		end
	end
end


function CooldownFrames.updatePrimaryBarColor(pGroupTable, pCooldownName)
	local unitType, ignoreInterval = Spells.unitTypeIndexByCooldownName[pCooldownName], pGroupTable.tables.unitTableIndex[pCooldownName].cooldowns.tables[pCooldownName].ignoreInterval;
	if ( ignoreInterval and pGroupTable.options.row.ignoreInterval[unitType].enable ) then
		Tools.setBarColor(pGroupTable.frames.rows[pCooldownName..'_bar1'], pGroupTable.options.row.ignoreInterval[unitType].color);
	else
		Tools.setBarColor(pGroupTable.frames.rows[pCooldownName..'_bar1'], pGroupTable.options.row.bar1[unitType].color);
	end
end


function CooldownFrames.updateSecondaryBarColor(pGroupTable, pCooldownName)
	local unitType, ignoreInterval = Spells.unitTypeIndexByCooldownName[pCooldownName], pGroupTable.tables.unitTableIndex[pCooldownName].cooldowns.tables[pCooldownName].ignoreInterval;
	if ( unitType == 'pet' ) then
		if ( not ignoreInterval ) then
			Tools.setBarColor(pGroupTable.frames.rows[pCooldownName..'_bar2'], pGroupTable.options.row.bar2.pet.color);
		end
	else
		if ( not ignoreInterval ) then
			Tools.setBarColor(pGroupTable.frames.rows[pCooldownName..'_bar2'], pGroupTable.options.row.bar2.player.color);
		end
	end
end


function CooldownFrames.updateDividerColor(pGroupTable, pCooldownName)
	local unitType, ignoreInterval = Spells.unitTypeIndexByCooldownName[pCooldownName], pGroupTable.tables.unitTableIndex[pCooldownName].cooldowns.tables[pCooldownName].ignoreInterval;
	if ( unitType == 'pet' ) then
		if ( not ignoreInterval ) then
			Tools.setColor(pGroupTable.frames.rows[pCooldownName..'_divider'].texture, pGroupTable.options.row.divider.pet.color);
		end
	else
		if ( not ignoreInterval ) then
			Tools.setColor(pGroupTable.frames.rows[pCooldownName..'_divider'].texture, pGroupTable.options.row.divider.player.color);
		end
	end
end


function CooldownFrames.updateRowBackgroundColor(pGroupTable, pCooldownName)
	local unitType = Spells.unitTypeIndexByCooldownName[pCooldownName];
	if ( pGroupTable.options.row.background[unitType].showTexture ) then
		Tools.setBackdropColor(pGroupTable.frames.rows[pCooldownName..'_background'], pGroupTable.options.row.background[unitType].color);
	else
		Tools.setColor(pGroupTable.frames.rows[pCooldownName..'_background'].texture, pGroupTable.options.row.background[unitType].color);
	end
end


function CooldownFrames.updateClassIcon(pGroupTable)
	local currentFrame = pGroupTable.frames.header.class.currentFrame;
	if ( currentFrame ~= nil ) then
		pGroupTable.frames.header.class[currentFrame]:Hide();
		pGroupTable.frames.header.class.currentFrame = nil;
	end
	if ( pGroupTable.options.anchor.class.show and pGroupTable.unit.table ~= nil and pGroupTable.unit.table.class ~= nil ) then
		pGroupTable.frames.header.class[pGroupTable.unit.table.class]:Show();
		pGroupTable.frames.header.class.currentFrame = pGroupTable.unit.table.class;
	end
end


function CooldownFrames.updateSpecIcon(pGroupTable)
	local currentFrame = pGroupTable.frames.header.spec.currentFrame;
	if ( currentFrame ~= nil ) then
		pGroupTable.frames.header.spec[currentFrame]:Hide();
		pGroupTable.frames.header.spec.currentFrame = nil;
	end
	if ( pGroupTable.options.anchor.spec.show and pGroupTable.unit.table ~= nil and pGroupTable.unit.table.spec ~= nil ) then
		local frameName = pGroupTable.unit.table.class..'_'..pGroupTable.unit.table.spec;
		pGroupTable.frames.header.spec[frameName]:Show();
		pGroupTable.frames.header.spec.currentFrame = frameName;
	end
end


function CooldownFrames.updatePetIcon(pGroupTable)
	local currentFrame = pGroupTable.frames.header.pet.currentFrame;
	if ( currentFrame ~= nil ) then
		pGroupTable.frames.header.pet[currentFrame]:Hide();
		pGroupTable.frames.header.pet.currentFrame = nil;
	end
	if ( pGroupTable.options.anchor.pet.show and pGroupTable.unit.table ~= nil and pGroupTable.unit.table.petLinkTable ~= nil ) then
		if ( pGroupTable.unit.table.petLinkTable.class ~= nil ) then
			pGroupTable.frames.header.pet[pGroupTable.unit.table.petLinkTable.class]:Show();
			pGroupTable.frames.header.pet.currentFrame = pGroupTable.unit.table.petLinkTable.class;
		else
			pGroupTable.frames.header.pet.generic:Show();
			pGroupTable.frames.header.pet.currentFrame = 'generic';
		end
	end
end


function CooldownFrames.updateUnitFramePositions(pGroupTable)
	-- 3 boolean properties, 2^3 (8) possible outcomes
	if ( pGroupTable.options.anchor.class.show ) then
		for i, className in pairs ( pGroupTable.frames.header.class.frameArray ) do
			pGroupTable.frames.header.class[className]:ClearAllPoints();
			pGroupTable.frames.header.class[className]:SetPoint('TOPLEFT', pGroupTable.frames.header.anchor, 'TOPLEFT');
		end
		if ( pGroupTable.options.anchor.spec.show ) then
			for i, specIconId in pairs ( pGroupTable.frames.header.spec.frameArray ) do
				pGroupTable.frames.header.spec[specIconId]:ClearAllPoints();
				pGroupTable.frames.header.spec[specIconId]:SetPoint('LEFT', pGroupTable.frames.header.class[pGroupTable.frames.header.class.frameArray[1]], 'RIGHT');
			end
			if ( pGroupTable.options.anchor.pet.show ) then
				for i, className in pairs ( pGroupTable.frames.header.pet.frameArray ) do
					pGroupTable.frames.header.pet[className]:ClearAllPoints();
					pGroupTable.frames.header.pet[className]:SetPoint('LEFT', pGroupTable.frames.header.spec[pGroupTable.frames.header.spec.frameArray[1]], 'RIGHT');
				end
			end			
		else
			if ( pGroupTable.options.anchor.pet.show ) then
				for i, className in pairs ( pGroupTable.frames.header.pet.frameArray ) do
					pGroupTable.frames.header.pet[className]:ClearAllPoints();
					pGroupTable.frames.header.pet[className]:SetPoint('LEFT', pGroupTable.frames.header.class[pGroupTable.frames.header.class.frameArray[1]], 'RIGHT');
				end
			end
		end
	else
		if ( pGroupTable.options.anchor.spec.show ) then
			for i, specIconId in pairs ( pGroupTable.frames.header.spec.frameArray ) do
				pGroupTable.frames.header.spec[specIconId]:ClearAllPoints();
				pGroupTable.frames.header.spec[specIconId]:SetPoint('TOPLEFT', pGroupTable.frames.header.anchor, 'TOPLEFT');
			end
			if ( pGroupTable.options.anchor.pet.show ) then
				for i, className in pairs ( pGroupTable.frames.header.pet.frameArray ) do
					pGroupTable.frames.header.pet[className]:ClearAllPoints();
					pGroupTable.frames.header.pet[className]:SetPoint('LEFT', pGroupTable.frames.header.spec[pGroupTable.frames.header.spec.frameArray[1]], 'RIGHT');
				end
			end
		else
			if ( pGroupTable.options.anchor.pet.show ) then
				for i, className in pairs ( pGroupTable.frames.header.pet.frameArray ) do
					pGroupTable.frames.header.pet[className]:ClearAllPoints();
					pGroupTable.frames.header.pet[className]:SetPoint('TOPLEFT', pGroupTable.frames.header.anchor, 'TOPLEFT');
				end
			end
		end
	end
end


function CooldownFrames.updateUnitNamePosition(pGroupTable)
	if ( pGroupTable.unit.name ~= nil ) then
		pGroupTable.frames.header.name:SetText(pGroupTable.unit.name);
		if ( pGroupTable.unit.server ~= nil ) then				
			pGroupTable.frames.header.server:SetText('-'..pGroupTable.unit.server);
		else
			pGroupTable.frames.header.server:SetText('');
		end
		if ( pGroupTable.options.anchor.name.show ) then
			pGroupTable.frames.header.name:Show(); -- Must be shown before updating text dimensions or else the width will be off
			Tools.updateTextDimensions(pGroupTable.frames.header.name, true, true);
			local anchorWidth = pGroupTable.options.anchor.background.width - (pGroupTable.options.row.textIndent * 2);				
			local nameWidth = pGroupTable.frames.header.name:GetWidth();
			if ( nameWidth < anchorWidth ) then
				if ( pGroupTable.options.anchor.server.show ) then
					pGroupTable.frames.header.server:Show(); -- Must be shown before updating text dimensions or else the width will be off
					Tools.updateTextDimensions(pGroupTable.frames.header.server, true, true);
					local serverWidth = pGroupTable.frames.header.server:GetWidth();
					if ( nameWidth + serverWidth < anchorWidth ) then
						pGroupTable.frames.header.name:SetPoint('BOTTOMLEFT', pGroupTable.frames.header.anchor, 'BOTTOMLEFT', ((anchorWidth-(nameWidth+serverWidth))/2)+pGroupTable.options.row.textIndent, pGroupTable.options.row.textIndent);
					else
						pGroupTable.frames.header.name:SetPoint('BOTTOMLEFT', pGroupTable.frames.header.anchor, 'BOTTOMLEFT', pGroupTable.options.row.textIndent, pGroupTable.options.row.textIndent);
						pGroupTable.frames.header.server:SetWidth(anchorWidth - nameWidth);
					end	
				else
					pGroupTable.frames.header.name:SetPoint('BOTTOMLEFT', pGroupTable.frames.header.anchor, 'BOTTOMLEFT', ((anchorWidth-nameWidth)/2)+pGroupTable.options.row.textIndent, pGroupTable.options.row.textIndent);
					pGroupTable.frames.header.server:Hide();	
				end
			else
				pGroupTable.frames.header.name:SetWidth(anchorWidth);
				pGroupTable.frames.header.name:SetPoint('BOTTOMLEFT', pGroupTable.frames.header.anchor, 'BOTTOMLEFT', pGroupTable.options.row.textIndent, pGroupTable.options.row.textIndent);		
				pGroupTable.frames.header.server:Hide();
			end
		else
			pGroupTable.frames.header.name:Hide();
			pGroupTable.frames.header.server:Hide();
		end
	else
		pGroupTable.frames.header.name:Hide();
		pGroupTable.frames.header.server:Hide();
	end
end


function CooldownFrames.getBarWidth(pGroupTable, pUnitType)
	if ( pGroupTable.options.row.icon[pUnitType].show ) then
		return pGroupTable.options.row.background[pUnitType].width - pGroupTable.options.row.icon[pUnitType].length;
	else
		return pGroupTable.options.row.background[pUnitType].width;
	end
end


function CooldownFrames.getCooldownExpirationTimestamp(pGroupTable, pUnitTable, pCooldownName)
	local cooldownTable, beginTimestamp, endpoint1Timestamp, endpoint2Timestamp = pUnitTable.cooldowns.tables[pCooldownName], pUnitTable.cooldowns.timestamps[pCooldownName];
	if ( pGroupTable.options.endpoint.endPoint1 == 'interval' ) then
		if ( pUnitTable.cooldowns.maxDurations[pCooldownName] ~= nil ) then
			endpoint1Timestamp = pUnitTable.cooldowns.maxDurations[pCooldownName];
		else
			endpoint1Timestamp = cooldownTable.maxDuration or cooldownTable.baseDuration;
		end
		if ( pGroupTable.options.endpoint.endPoint2 == 'longest' ) then
			endpoint2Timestamp = cooldownTable.maxDuration or cooldownTable.baseDuration;
		end
	elseif ( pGroupTable.options.endpoint.endPoint1 == 'longest' ) then
		endpoint1Timestamp = cooldownTable.maxDuration or cooldownTable.baseDuration;
	else
		-- default endpoint 1 = shortest
		endpoint1Timestamp = cooldownTable.minDuration or cooldownTable.baseDuration;
		if ( pGroupTable.options.endpoint.endPoint2 == 'longest' ) then
			endpoint2Timestamp = cooldownTable.maxDuration or cooldownTable.baseDuration;
		elseif ( pGroupTable.options.endpoint.endPoint2 == 'disable' ) then
			-- do nothing, since interval is default
		else
			-- default endpoint 2 = interval
			if ( pUnitTable.cooldowns.maxDurations[pCooldownName] ~= nil ) then
				endpoint2Timestamp = pUnitTable.cooldowns.maxDurations[pCooldownName];
			else
				endpoint2Timestamp = cooldownTable.maxDuration or cooldownTable.baseDuration;
			end
		end
	end
	endpoint1Timestamp = endpoint1Timestamp + beginTimestamp;
	if ( endpoint2Timestamp ~= nil ) then
		endpoint2Timestamp = endpoint2Timestamp + beginTimestamp;
		if ( endpoint1Timestamp >= endpoint2Timestamp ) then
			return endpoint1Timestamp, nil;
		else
			return endpoint2Timestamp, endpoint1Timestamp;
		end
	else
		return endpoint1Timestamp, nil;
	end
end


function CooldownFrames.updateGuidIndex(pFrameObjectName, pUnitGuid)
	local oldGuid = CooldownFrames.guidIndexByGroupName[pFrameObjectName];
	if ( oldGuid ~= nil ) then
		CooldownFrames.guidIndexByGroupName[pFrameObjectName] = nil;
		local guidFound;
		for groupName, guid in pairs ( CooldownFrames.guidIndexByGroupName ) do
			if ( guid == oldGuid ) then
				guidFound = true;
				break;
			end
		end
		if ( not guidFound ) then
			CooldownFrames.groupNameIndexByGuid[oldGuid] = nil;
		end
	end
	
	if ( pUnitGuid ~= nil ) then
		CooldownFrames.guidIndexByGroupName[pFrameObjectName] = pUnitGuid;
		CooldownFrames.groupNameIndexByGuid[pUnitGuid] = pFrameObjectName;
	end
end


function CooldownFrames.updatePetLinkGuidIndex(pFrameObjectName, pUnitGuid)
	local oldGuid = CooldownFrames.petLinkGuidIndexByGroupName[pFrameObjectName];
	if ( oldGuid ~= nil ) then
		CooldownFrames.petLinkGuidIndexByGroupName[pFrameObjectName] = nil;
		local guidFound;
		for groupName, guid in pairs ( CooldownFrames.petLinkGuidIndexByGroupName ) do
			if ( guid == oldGuid ) then
				guidFound = true;
				break;
			end
		end
		if ( not guidFound ) then
			CooldownFrames.groupNameIndexByPetLinkGuid[oldGuid] = nil;
		end
	end
	
	if ( pUnitGuid ~= nil ) then
		CooldownFrames.petLinkGuidIndexByGroupName[pFrameObjectName] = pUnitGuid;
		CooldownFrames.groupNameIndexByPetLinkGuid[pUnitGuid] = pFrameObjectName;
	end
end


function CooldownFrames.buildUnitIdFrame(pUnitId)
	
	-- Make sure the unit ID is valid
	if ( UnitGUID(pUnitId) == nil ) then
		return nil;
	end
	
	if ( #avaliableExtraFrameArray > 0 ) then
		local groupName = avaliableExtraFrameArray[1];
		CooldownFrames.setGroupUnit(groupName, pUnitId);
		table.remove(avaliableExtraFrameArray, 1);
		unavaliableExtraFrameIndex[groupName] = 1;
		unavaliableExtraFrameCounter = unavaliableExtraFrameCounter + 1;
	else
		extraFrameCounter = extraFrameCounter + 1;
		local groupName = 'ef'..extraFrameCounter;
		CooldownFrames.buildCooldownFrame(groupName, 'ef', false, true);
		CooldownFrames.setGroupUnit(groupName, pUnitId);
		unavaliableExtraFrameIndex[groupName] = 1;
		unavaliableExtraFrameCounter = unavaliableExtraFrameCounter + 1;
	end
	
	return true;
end


function CooldownFrames.showTestUnit(pGroupName)
	local groupTable = cf[pGroupName];
	testUnit.cooldowns.timestamps.player_test_base_bar1 = WOOGLED.time.now - 2000;
	testUnit.cooldowns.timestamps.player_test_base_bar2 = WOOGLED.time.now - 1000;
	testUnit.cooldowns.timestamps.player_test_base_ignoreInterval = WOOGLED.time.now;
	testPet.cooldowns.timestamps.pet_test_base_bar1 = WOOGLED.time.now - 2001;
	testPet.cooldowns.timestamps.pet_test_base_bar2 = WOOGLED.time.now - 1001;
	testPet.cooldowns.timestamps.pet_test_base_ignoreInterval = WOOGLED.time.now;
	groupTable.unit.guid = testUnit.guid;
	groupTable.unit.type = 'player';
	groupTable.unit.table = testUnit;
	CooldownFrames.updateGuidIndex(pGroupName, testUnit.guid);
	CooldownFrames.updateGroupTables(groupTable);
	CooldownFrames.updateUnitIcons(groupTable);
	groupTable.unit.name, groupTable.unit.server = testUnit.name, testUnit.server;
	CooldownFrames.setVisibility(true, pGroupName);
	CooldownFrames.updateUnitNamePosition(groupTable); -- has to be called after showing the frame, or else name/server text will be truncated
end


function CooldownFrames.setGroupUnit(pGroupName, pUnitId)
	local groupTable = cf[pGroupName];
	if ( pUnitId ~= nil ) then
		local guid = UnitGUID(pUnitId);
		if ( guid ~= nil ) then
			if ( guid ~= groupTable.unit.guid ) then
				groupTable.unit.guid = guid;
				groupTable.unit.type, groupTable.unit.table = Units.getUnitInfo(guid);
				CooldownFrames.updateGuidIndex(pGroupName, guid);
				
				local petLinkGuid;
				if ( groupTable.unit.table ~= nil and groupTable.unit.table.petLinkTable ~= nil ) then
					petLinkGuid = groupTable.unit.table.petLinkTable.guid;
				end
				CooldownFrames.updatePetLinkGuidIndex(pGroupName, petLinkGuid);
				
				CooldownFrames.updateGroupTables(groupTable);
				CooldownFrames.updateUnitIcons(groupTable);
			end
			groupTable.unit.name, groupTable.unit.server = UnitName(pUnitId);
			CooldownFrames.setVisibility(true, pGroupName);
			CooldownFrames.updateUnitNamePosition(groupTable); -- has to be called after showing the frame, or else name/server text will be truncated
		elseif ( not cf[pGroupName].options.stickyTargeting ) then
			groupTable.unit.guid = nil;
			groupTable.unit.type = nil;
			groupTable.unit.table = nil;
			groupTable.unit.name, groupTable.unit.server = nil, nil;
			CooldownFrames.updateGuidIndex(pGroupName, nil);
			CooldownFrames.updatePetLinkGuidIndex(pGroupName, nil);
			CooldownFrames.updateUnitNamePosition(groupTable);
			CooldownFrames.updateGroupTables(groupTable);
			CooldownFrames.updateUnitIcons(groupTable);
			CooldownFrames.setVisibility(false, pGroupName);
		end
	else
		groupTable.unit.guid = nil;
		groupTable.unit.type = nil;
		groupTable.unit.table = nil;
		groupTable.unit.name, groupTable.unit.server = nil, nil;
		CooldownFrames.updateGuidIndex(pGroupName, nil);
		CooldownFrames.updatePetLinkGuidIndex(pGroupName, nil);
		CooldownFrames.updateUnitNamePosition(groupTable);
		CooldownFrames.updateGroupTables(groupTable);
		CooldownFrames.updateUnitIcons(groupTable);
		CooldownFrames.setVisibility(false, pGroupName);
	end
end


function CooldownFrames.onCooldownFinish(pGroupTable, pCooldownName, pIndex)
	table.remove(pGroupTable.tables.cooldownNameArray, pIndex);
	pGroupTable.tables.unitTableIndex[pCooldownName]= nil;
	pGroupTable.frames.rows[pCooldownName..'_background']:Hide();
	CooldownFrames.updateRowPositions(pGroupTable);
end


function CooldownFrames.closeExtraFrame(pGroupName)
	unavaliableExtraFrameCounter = unavaliableExtraFrameCounter - 1;
	CooldownFrames.setGroupUnit(pGroupName, nil);
	unavaliableExtraFrameIndex[pGroupName] = nil;
	table.insert(avaliableExtraFrameArray, pGroupName);
end


-- [BLIZZARD EVENTS] --


function CooldownFrames.onTargetChange()
	if ( not cf.tf.pinned ) then
		CooldownFrames.setGroupUnit('tf', 'target');
	end
end


function CooldownFrames.onFocusChange()
	CooldownFrames.setGroupUnit('ff', 'focus');
end


-- [UNIT DATABASE EVENTS] --


function CooldownFrames.onUnitAdd(pUnitTable)
	if ( CooldownFrames.groupNameIndexByGuid[pUnitTable.guid] ~= nil ) then
		for groupName, groupTable in pairs ( cf ) do
			if ( groupTable.unit.guid == pUnitTable.guid ) then
				groupTable.unit.table = pUnitTable;
			end
		end
	end
end


function CooldownFrames.onCooldownReset(pGuid)
	if ( CooldownFrames.groupNameIndexByGuid[pGuid] ~= nil ) then
		for groupName, groupTable in pairs ( cf ) do
			if ( groupTable.unit.guid == pGuid ) then
				CooldownFrames.updateGroupTables(groupTable);
			end
		end
	end
end


function CooldownFrames.onRecordCooldown(pGuid, pCooldownName)
	if ( CooldownFrames.groupNameIndexByGuid[pGuid] ~= nil ) then
		for groupName, groupTable in pairs ( cf ) do
			if ( groupTable.unit.guid == pGuid ) then
				local expirationTimestamp, midpointTimestamp = CooldownFrames.getCooldownExpirationTimestamp(groupTable, groupTable.unit.table, pCooldownName);
				CooldownFrames.addCooldownToDisplay(groupTable, groupTable.unit.table, pCooldownName, expirationTimestamp, midpointTimestamp);
				CooldownFrames.updateRowPositions(groupTable);
			end
		end
	end
	if ( CooldownFrames.groupNameIndexByPetLinkGuid[pGuid] ~= nil ) then
		for groupName, groupTable in pairs ( cf ) do
			if ( groupTable.options.showPet and groupTable.unit.table ~= nil and groupTable.unit.table.petLinkTable ~= nil and groupTable.unit.table.petLinkTable.guid == pGuid ) then
				local expirationTimestamp, midpointTimestamp = CooldownFrames.getCooldownExpirationTimestamp(groupTable, groupTable.unit.table.petLinkTable, pCooldownName);
				CooldownFrames.addCooldownToDisplay(groupTable, groupTable.unit.table.petLinkTable, pCooldownName, expirationTimestamp, midpointTimestamp);
				CooldownFrames.updateRowPositions(groupTable);
			end
		end
	end
end


function CooldownFrames.onDiscoverClass(pGuid)
	-- Updates class or pet icon.
	if ( CooldownFrames.groupNameIndexByGuid[pGuid] ~= nil ) then
		for groupName, groupTable in pairs ( cf ) do
			if ( groupTable.unit.guid == pGuid ) then
				CooldownFrames.updateClassIcon(groupTable);
			end
		end
	end
	if ( CooldownFrames.groupNameIndexByPetLinkGuid[pGuid] ~= nil ) then
		for groupName, groupTable in pairs ( cf ) do
			if ( groupTable.unit.table ~= nil and groupTable.unit.table.petLinkTable ~= nil and groupTable.unit.table.petLinkTable.guid == pGuid ) then
				CooldownFrames.updatePetIcon(groupTable);
			end
		end
	end
end


function CooldownFrames.onDiscoverSpec(pGuid)
	if ( CooldownFrames.groupNameIndexByGuid[pGuid] ~= nil ) then
		for groupName, groupTable in pairs ( cf ) do
			if ( groupTable.unit.guid == pGuid ) then
				CooldownFrames.updateSpecIcon(groupTable);
			end
		end
	end
end


function CooldownFrames.onDiscoverPet(pGuid)
	if ( CooldownFrames.groupNameIndexByGuid[pGuid] ~= nil or CooldownFrames.groupNameIndexByPetLinkGuid[pGuid] ) then
		for groupName, groupTable in pairs ( cf ) do
			if ( groupTable.unit.table ~= nil and (groupTable.unit.table.guid == pGuid or (groupTable.unit.table.petLinkTable ~= nil and groupTable.unit.table.petLinkTable.guid == pGuid)) ) then
				CooldownFrames.updatePetIcon(groupTable);
				CooldownFrames.updatePetLinkGuidIndex(groupName, groupTable.unit.table.petLinkTable.guid);
				if ( groupTable.unit.table.unitType == 'player' ) then
					CooldownFrames.updateGroupTables(groupTable);
				end
			end
		end
	end
end


-- [FRAME SPECIFIC] --


function CooldownFrames.setVisibility(pState, pGroupName)
	if ( pState == nil ) then
		-- nil not accepted
	elseif ( pState ) then
		cf[pGroupName].frames.header.anchor:Show();
	else
		cf[pGroupName].frames.header.anchor:Hide();
	end
	--OptionsFrame.groupFrames[pOptionsName..'_show_button']:SetChecked(pState);
end


function CooldownFrames.setPin(pState, pGroupName)
	if ( pState == nil ) then
		pState = cf[pGroupName].pinned;
	end
	cf[pGroupName].pinned = pState;
	if ( pState == true ) then
		cf[pGroupName].frames.header.pin:Hide();
		cf[pGroupName].frames.header.unpin:Show();
	else
		cf[pGroupName].frames.header.unpin:Hide();
		cf[pGroupName].frames.header.pin:Show();
		CooldownFrames.onTargetChange();
	end
end


function CooldownFrames.setStickyTargeting(pState, pGroupName)
	if ( pState == nil ) then
		pState = cf[pGroupName].options.stickyTargeting;
	end
	cf[pGroupName].options.stickyTargeting = pState;
	CooldownFrames.onTargetChange();
end


-- [GENERAL FRAME] --


function CooldownFrames.setLock(pState, pOptionsName)
	local groupsTable = groupsTableIndexByGroupName[pOptionsName];
	for i, groupName in pairs ( groupsTable ) do
		if ( pState == nil ) then
			if ( cf[groupName].options.lock ) then
				cf[groupName].frames.header.anchor:RegisterForDrag();
			else
				cf[groupName].frames.header.anchor:RegisterForDrag('LeftButton');
			end
		elseif ( pState ) then
			cf[groupName].frames.header.anchor:RegisterForDrag();
			cf[groupName].options.lock = true;
		else
			cf[groupName].frames.header.anchor:RegisterForDrag('LeftButton');
			cf[groupName].options.lock = false;
		end
		--OptionsFrame.groupFrames[pOptionsName..'_lock_button']:SetChecked(cf[groupName].options.lock);
	end
end


function CooldownFrames.setScale(pScale, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pScale = tonumber(pScale);
	if ( pScale == nil or pScale <= 0 ) then
		pScale = optionsTable.scale;
	end
	optionsTable.scale = pScale;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.anchor:SetScale(pScale);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_scale_editBox']:SetText(pScale);
end


function CooldownFrames.showPet(pState, pOptionsName)
	local groupsTable = groupsTableIndexByGroupName[pOptionsName];
	for i, groupName in pairs ( groupsTable ) do
		if ( pState == nil ) then
			-- handled by updateGroupTables()
		elseif ( pState ) then
			cf[groupName].options.showPet = true;
		else
			cf[groupName].options.showPet = false;
		end
		CooldownFrames.updateGroupTables(cf[groupName]);
	end
end


-- [ENDPOINTS] --


function CooldownFrames.endpoint_setEndPoint1(pPoint, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pPoint = pPoint or optionsTable.endpoint.endPoint1;
	optionsTable.endpoint.endPoint1 = pPoint;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateGroupTables(cf[frameName]);
	end
	if ( pPoint == 'interval' ) then
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint1Shortest_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint1Interval_button']:SetChecked(true);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint1Longest_button']:SetChecked(false);
	elseif ( pPoint == 'longest' ) then
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint1Shortest_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint1Interval_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint1Longest_button']:SetChecked(true);
	else
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint1Shortest_button']:SetChecked(true);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint1Interval_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint1Longest_button']:SetChecked(false);
	end
end


function CooldownFrames.endpoint_setEndPoint2(pPoint, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pPoint = pPoint or optionsTable.endpoint.endPoint2;
	optionsTable.endpoint.endPoint2 = pPoint;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateGroupTables(cf[frameName]);
	end
	-- Update Options Frame
	if ( pPoint == 'disable' ) then
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint2Disable_button']:SetChecked(true);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint2Interval_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint2Longest_button']:SetChecked(false);
	elseif ( pPoint == 'longest' ) then
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint2Disable_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint2Interval_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint2Longest_button']:SetChecked(true);
	else
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint2Disable_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint2Interval_button']:SetChecked(true);
		OptionsFrame.groupFrames[pOptionsName..'_endpoint_endpoint2Longest_button']:SetChecked(false);
	end
end


--[ANCHOR BACKGROUND]--


function CooldownFrames.anchor_setWidth(pWidth, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pWidth = tonumber(pWidth);
	if ( pWidth == nil or pWidth <= 0 ) then
		pWidth = optionsTable.anchor.background.width;
	end
	optionsTable.anchor.background.width = pWidth;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.anchor:SetWidth(pWidth);
		CooldownFrames.updateUnitNamePosition(cf[frameName]);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_width_editBox']:SetText(pWidth);
end


function CooldownFrames.anchor_setHeight(pHeight, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pHeight = tonumber(pHeight);
	if ( pHeight == nil or pHeight <= 0 ) then		
		pHeight = optionsTable.anchor.background.height;
	end
	optionsTable.anchor.background.height = pHeight;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.anchor:SetHeight(pHeight);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_height_editBox']:SetText(pHeight);
end


function CooldownFrames.anchor_setBackgroundColor(pR, pG, pB, pA, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.anchor.background.r;
	pG = pG or optionsTable.anchor.background.g;
	pB = pB or optionsTable.anchor.background.b;
	pA = pA or optionsTable.anchor.background.a;
	optionsTable.anchor.background.r = pR;
	optionsTable.anchor.background.g = pG;
	optionsTable.anchor.background.b = pB;
	optionsTable.anchor.background.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( optionsTable.anchor.background.showTexture ) then
			cf[frameName].frames.header.anchor:SetBackdropColor(pR, pG, pB, pA);
		else
			cf[frameName].frames.header.anchor.texture:SetTexture(pR, pG, pB, pA);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_anchorBackgroundColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


function CooldownFrames.anchor_setBackgroundTextureVisibility(pChecked, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.anchor.background.showTexture;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_texture_anchor_backgroundEnableTexture_button']:SetChecked(pChecked);
	end
	optionsTable.anchor.background.showTexture = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( pChecked ) then
			cf[frameName].frames.header.anchor.texture:SetTexture(nil);
			cf[frameName].frames.header.anchor:SetBackdrop({bgFile = optionsTable.anchor.background.texture});
			Tools.setBackdropColor(cf[frameName].frames.header.anchor, optionsTable.anchor.background);
		else
			Tools.setColor(cf[frameName].frames.header.anchor.texture, optionsTable.anchor.background);
			cf[frameName].frames.header.anchor:SetBackdrop(nil);
			cf[frameName].frames.header.anchor:SetBackdropColor(nil);
		end
	end
end


function CooldownFrames.anchor_setBackgroundTexture(pTexturePath, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pTexturePath = pTexturePath or optionsTable.anchor.background.texture;
	optionsTable.anchor.background.texture = pTexturePath;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( optionsTable.anchor.background.showTexture ) then
			cf[frameName].frames.header.anchor:SetBackdrop({bgFile = pTexturePath});
			Tools.setBackdropColor(cf[frameName].frames.header.anchor, optionsTable.anchor.background);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_anchor_backgroundPath_editBox']:SetText(pTexturePath);
end


-- [CLASS ICON] --


function CooldownFrames.anchor_setClassVisibility(pChecked, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.anchor.class.show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_anchor_classShow_button']:SetChecked(pChecked);
	end
	optionsTable.anchor.class.show = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateUnitFramePositions(cf[frameName]);
		CooldownFrames.updateClassIcon(cf[frameName]);
	end
end


function CooldownFrames.anchor_setClassLength(pLength, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pLength = tonumber(pLength);
	if ( pLength == nil or pLength <= 0 ) then
		pLength = optionsTable.anchor.class.length;
	end
	optionsTable.anchor.class.length = pLength;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, className in pairs ( cf[frameName].frames.header.class.frameArray ) do
			cf[frameName].frames.header.class[className]:SetWidth(pLength);
			cf[frameName].frames.header.class[className]:SetHeight(pLength);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_classLength_editBox']:SetText(pLength);
end


function CooldownFrames.anchor_setClassAlpha(pAlpha, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pAlpha = tonumber(pAlpha);
	if ( pAlpha == nil or pAlpha < 0 or pAlpha > 1 ) then
		pAlpha = optionsTable.anchor.class.a;
	end
	optionsTable.anchor.class.a = pAlpha;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, className in pairs ( cf[frameName].frames.header.class.frameArray ) do
			cf[frameName].frames.header.class[className]:SetAlpha(pAlpha);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_classAlpha_editBox']:SetText(pAlpha);
end


-- [SPEC ICON] --


function CooldownFrames.anchor_setSpecVisibility(pChecked, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.anchor.spec.show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_anchor_specShow_button']:SetChecked(pChecked);
	end
	optionsTable.anchor.spec.show = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateUnitFramePositions(cf[frameName]);
		CooldownFrames.updateSpecIcon(cf[frameName]);
	end
end


function CooldownFrames.anchor_setSpecLength(pLength, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pLength = tonumber(pLength);
	if ( pLength == nil or pLength <= 0 ) then
		pLength = optionsTable.anchor.spec.length;
	end
	optionsTable.anchor.spec.length = pLength;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, iconId in pairs ( cf[frameName].frames.header.spec.frameArray ) do
			cf[frameName].frames.header.spec[iconId]:SetWidth(pLength);
			cf[frameName].frames.header.spec[iconId]:SetHeight(pLength);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_specLength_editBox']:SetText(pLength);
end


function CooldownFrames.anchor_setSpecAlpha(pAlpha, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pAlpha = tonumber(pAlpha);
	if ( pAlpha == nil or pAlpha < 0 or pAlpha > 1 ) then
		pAlpha = optionsTable.anchor.spec.a;
	end
	optionsTable.anchor.spec.a = pAlpha;
	-- Update Cooldown Frames	
	for i, frameName in pairs ( frameNameArray ) do
		for i, iconId in pairs ( cf[frameName].frames.header.spec.frameArray ) do
			cf[frameName].frames.header.spec[iconId]:SetAlpha(pAlpha);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_specAlpha_editBox']:SetText(pAlpha);
end


-- [PET LINK ICONS] --


function CooldownFrames.anchor_setPetVisibility(pChecked, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.anchor.pet.show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_anchor_petShow_button']:SetChecked(pChecked);
	end
	optionsTable.anchor.pet.show = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateUnitFramePositions(cf[frameName]);
		CooldownFrames.updatePetIcon(cf[frameName]);
	end
end


function CooldownFrames.anchor_setPetLength(pLength, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pLength = tonumber(pLength);
	if ( pLength == nil or pLength <= 0 ) then
		pLength = optionsTable.anchor.pet.length;
	end
	optionsTable.anchor.pet.length = pLength;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, className in pairs ( cf[frameName].frames.header.pet.frameArray ) do
			cf[frameName].frames.header.pet[className]:SetWidth(pLength);
			cf[frameName].frames.header.pet[className]:SetHeight(pLength);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_petLength_editBox']:SetText(pLength);
end


function CooldownFrames.anchor_setPetAlpha(pAlpha, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pAlpha = tonumber(pAlpha);
	if ( pAlpha == nil or pAlpha < 0 or pAlpha > 1 ) then
		pAlpha = optionsTable.anchor.pet.a;
	end
	optionsTable.anchor.pet.a = pAlpha;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, className in pairs ( cf[frameName].frames.header.pet.frameArray ) do
			cf[frameName].frames.header.pet[className]:SetAlpha(pAlpha);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_petAlpha_editBox']:SetText(pAlpha);
end


-- [PIN BUTTON] --


function CooldownFrames.anchor_setPinVisibility(pChecked, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.anchor.pin.show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_anchor_pinShow_button']:SetChecked(pChecked);
	end
	optionsTable.anchor.pin.show = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.setPin(false, frameName);
		if ( not pChecked ) then
			cf[frameName].frames.header.pin:Hide();
			cf[frameName].frames.header.unpin:Hide();
		end
	end
end


function CooldownFrames.anchor_setPinLength(pLength, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pLength = tonumber(pLength);
	if ( pLength == nil or pLength <= 0 ) then
		pLength = optionsTable.anchor.pin.length;
	end
	optionsTable.anchor.pin.length = pLength;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.pin:SetWidth(pLength);
		cf[frameName].frames.header.pin:SetHeight(pLength);
		cf[frameName].frames.header.unpin:SetWidth(pLength);
		cf[frameName].frames.header.unpin:SetHeight(pLength);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_pinLength_editBox']:SetText(pLength);
end


function CooldownFrames.anchor_setPinAlpha(pAlpha, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pAlpha = tonumber(pAlpha);
	if ( pAlpha == nil or pAlpha < 0 or pAlpha > 1 ) then
		pAlpha = optionsTable.anchor.pin.a;
	end
	optionsTable.anchor.pin.a = pAlpha;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.pin:SetAlpha(pAlpha);
		cf[frameName].frames.header.unpin:SetAlpha(pAlpha);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_pinAlpha_editBox']:SetText(pAlpha);
end


-- [CLOSE BUTTON] --


function CooldownFrames.anchor_setCloseLength(pLength, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pLength = tonumber(pLength);
	if ( pLength == nil or pLength <= 0 ) then
		pLength = optionsTable.anchor.close.length;
	end
	optionsTable.anchor.close.length = pLength;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.close:SetWidth(pLength);
		cf[frameName].frames.header.close:SetHeight(pLength);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_closeLength_editBox']:SetText(pLength);
end


function CooldownFrames.anchor_setCloseAlpha(pAlpha, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pAlpha = tonumber(pAlpha);
	if ( pAlpha == nil or pAlpha < 0 or alpha > 1 ) then
		pAlpha = optionsTable.anchor.close.a;
	end
	optionsTable.anchor.close.a = pAlpha;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.close:SetAlpha(pAlpha);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_anchor_closeAlpha_editBox']:SetText(pAlpha);
end


-- [UNIT NAME] --


function CooldownFrames.anchor_setNameVisibility(pChecked, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.anchor.name.show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_anchor_nameShow_button']:SetChecked(pChecked);
	end
	optionsTable.anchor.name.show = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateUnitNamePosition(cf[frameName]);
	end
end


function CooldownFrames.anchor_setNameColor(pR, pG, pB, pA, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.anchor.name.r;
	pG = pG or optionsTable.anchor.name.g;
	pB = pB or optionsTable.anchor.name.b;
	pA = pA or optionsTable.anchor.name.a;
	optionsTable.anchor.name.r = pR;
	optionsTable.anchor.name.g = pG;
	optionsTable.anchor.name.b = pB;
	optionsTable.anchor.name.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.name:SetTextColor(pR, pG, pB, pA);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_anchor_nameColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


function CooldownFrames.anchor_setNameFontSize(pFontSize, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pFontSize = tonumber(pFontSize);
	if ( pFontSize == nil or pFontSize <= 0 ) then
		pFontSize = optionsTable.anchor.name.fontSize;
	end
	optionsTable.anchor.name.fontSize = pFontSize;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.name:SetFont(optionsTable.anchor.name.font, pFontSize);
		CooldownFrames.updateUnitNamePosition(cf[frameName]);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_anchor_nameFontSize_editBox']:SetText(pFontSize);
end


function CooldownFrames.anchor_setNameFont(pFont, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pFont = pFont or optionsTable.anchor.name.font;
	optionsTable.anchor.name.font = pFont;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.name:SetFont(pFont, optionsTable.anchor.name.fontSize);
		CooldownFrames.updateUnitNamePosition(cf[frameName]);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_anchor_nameFont_editBox']:SetText(pFont);
end


-- [UNIT SERVER] --


function CooldownFrames.anchor_setServerVisibility(pChecked, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.anchor.server.show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_anchor_serverShow_button']:SetChecked(pChecked);
	end
	optionsTable.anchor.server.show = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateUnitNamePosition(cf[frameName]);
	end
end


function CooldownFrames.anchor_setServerColor(pR, pG, pB, pA, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.anchor.server.r;
	pG = pG or optionsTable.anchor.server.g;
	pB = pB or optionsTable.anchor.server.b;
	pA = pA or optionsTable.anchor.server.a;
	optionsTable.anchor.server.r = pR;
	optionsTable.anchor.server.g = pG;
	optionsTable.anchor.server.b = pB;
	optionsTable.anchor.server.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.server:SetTextColor(pR, pG, pB, pA);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_anchor_serverColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


function CooldownFrames.anchor_setServerFontSize(pFontSize, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pFontSize = tonumber(pFontSize);
	if ( pFontSize == nil or pFontSize <= 0 ) then
		pFontSize = optionsTable.anchor.server.fontSize;
	end
	optionsTable.anchor.server.fontSize = pFontSize;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.server:SetFont(optionsTable.anchor.server.font, pFontSize);
		CooldownFrames.updateUnitNamePosition(cf[frameName]);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_anchor_serverFontSize_editBox']:SetText(pFontSize);
end


function CooldownFrames.anchor_setServerFont(pFont, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pFont = pFont or optionsTable.anchor.server.font;
	optionsTable.anchor.server.font = pFont;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		cf[frameName].frames.header.server:SetFont(pFont, optionsTable.anchor.server.fontSize);
		CooldownFrames.updateUnitNamePosition(cf[frameName]);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_anchor_serverFont_editBox']:SetText(pFont);
end


-- [GENERAL ROW] --


function CooldownFrames.row_setAlignment(pDirection, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pDirection = pDirection or optionsTable.row.alignment;
	optionsTable.row.alignment = pDirection;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateRowPositions(cf[frameName]);
	end
	-- Update Options Frame
	if ( pDirection == 'left' ) then
		OptionsFrame.groupFrames[pOptionsName..'_row_alignLeft_button']:SetChecked(true);
		OptionsFrame.groupFrames[pOptionsName..'_row_alignCenter_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_row_alignRight_button']:SetChecked(false);
	elseif ( pDirection == 'right' ) then
		OptionsFrame.groupFrames[pOptionsName..'_row_alignLeft_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_row_alignCenter_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_row_alignRight_button']:SetChecked(true);
	else
		OptionsFrame.groupFrames[pOptionsName..'_row_alignLeft_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_row_alignCenter_button']:SetChecked(true);
		OptionsFrame.groupFrames[pOptionsName..'_row_alignRight_button']:SetChecked(false);
	end
end


function CooldownFrames.row_setBuildDirection(pDirection, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pDirection = pDirection or optionsTable.row.buildDirection;
	optionsTable.row.buildDirection = pDirection;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateRowPositions(cf[frameName]);
	end
	-- Update Options Frame
	if ( pDirection == 'up' ) then
		OptionsFrame.groupFrames[pOptionsName..'_row_buildUp_button']:SetChecked(true);
		OptionsFrame.groupFrames[pOptionsName..'_row_buildDown_button']:SetChecked(false);
	else
		OptionsFrame.groupFrames[pOptionsName..'_row_buildUp_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_row_buildDown_button']:SetChecked(true);
	end
end


function CooldownFrames.row_setStackDirection(pDirection, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pDirection = pDirection or optionsTable.row.stackDirection;
	optionsTable.row.stackDirection = pDirection;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateRowPositions(cf[frameName]);
	end
	-- Update Options Frame
	if ( pDirection == 'up' ) then
		OptionsFrame.groupFrames[pOptionsName..'_row_stackUp_button']:SetChecked(true);
		OptionsFrame.groupFrames[pOptionsName..'_row_stackDown_button']:SetChecked(false);
	else
		OptionsFrame.groupFrames[pOptionsName..'_row_stackUp_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_row_stackDown_button']:SetChecked(true);
	end
end


function CooldownFrames.row_setSortType(pSortType, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pSortType = pSortType or optionsTable.row.sortBy;
	optionsTable.row.sortBy = pSortType;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateRowPositions(cf[frameName]);
	end
	-- Update Options Frame
	if ( pSortType == 'event' ) then
		OptionsFrame.groupFrames[pOptionsName..'_row_sortTypeEvent_button']:SetChecked(true);
		OptionsFrame.groupFrames[pOptionsName..'_row_sortTypeCooldown_button']:SetChecked(false);
	else
		OptionsFrame.groupFrames[pOptionsName..'_row_sortTypeEvent_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_row_sortTypeCooldown_button']:SetChecked(true);
	end
end


function CooldownFrames.row_setSpacing(pSpacing, pOptionsName)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pSpacing = tonumber(pSpacing);
	if ( pSpacing == nil or pSpacing <= 0 ) then
		pSpacing = optionsTable.row.background.spacing;
	end
	optionsTable.row.background.spacing = pSpacing;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		CooldownFrames.updateRowPositions(cf[frameName]);
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_spacing_editBox']:SetText(pSpacing);
end


-- [ROW BACKGROUND]--


function CooldownFrames.row_setWidth(pWidth, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pWidth = tonumber(pWidth);
	if ( pWidth == nil or pWidth <= 0 ) then
		pWidth = optionsTable.row.background[pUnitType].width;
	end
	optionsTable.row.background[pUnitType].width = pWidth;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
			cf[frameName].frames.rows[cooldownName..'_background']:SetWidth(pWidth);
			if ( cf[frameName].tables.unitTableIndex[cooldownName] ~= nil and Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateStatusBarPositions(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_width_editBox']:SetText(pWidth);
end


function CooldownFrames.row_setHeight(pHeight, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pHeight = tonumber(pHeight);
	if ( pHeight == nil or pHeight <= 0 ) then
		pHeight = optionsTable.row.background[pUnitType].height;
	end
	optionsTable.row.background[pUnitType].height = pHeight;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
			cf[frameName].frames.rows[cooldownName..'_bar1']:SetHeight(pHeight);
			if ( not cooldownTable.ignoreInterval ) then
				cf[frameName].frames.rows[cooldownName..'_bar2']:SetHeight(pHeight);
				cf[frameName].frames.rows[cooldownName..'_divider']:SetHeight(pHeight);
			end
			cf[frameName].frames.rows[cooldownName..'_background']:SetHeight(pHeight);
			cf[frameName].frames.rows[cooldownName..'_background']:SetHeight(pHeight);		
		end
		
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_height_editBox']:SetText(pHeight);
end


function CooldownFrames.row_setBackgroundTextureVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.background[pUnitType].showTexture;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_backgroundEnableTexture_button']:SetChecked(pChecked);
	end
	optionsTable.row.background[pUnitType].showTexture = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( pChecked ) then
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				cf[frameName].frames.rows[cooldownName..'_background'].texture:SetTexture(nil);
				cf[frameName].frames.rows[cooldownName..'_background']:SetBackdrop({bgFile = optionsTable.row.background[pUnitType].texture});
				Tools.setBackdropColor(cf[frameName].frames.rows[cooldownName..'_background'], optionsTable.row.background[pUnitType].color);
			end
		else
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				Tools.setColor(cf[frameName].frames.rows[cooldownName..'_background'].texture, optionsTable.row.background[pUnitType].color);
				cf[frameName].frames.rows[cooldownName..'_background']:SetBackdrop(nil);
				cf[frameName].frames.rows[cooldownName..'_background']:SetBackdropColor(nil);
			end
		end
	end
end


function CooldownFrames.row_setBackgroundTexture(pTexturePath, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pTexturePath = pTexturePath or optionsTable.row.background[pUnitType].texture;
	optionsTable.row.background[pUnitType].texture = pTexturePath;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( optionsTable.row.background[pUnitType].showTexture ) then
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndex ) do
				cf[frameName].frames.rows[cooldownName..'_background']:SetBackdrop({bgFile = pTexturePath});
				Tools.setBackdropColor(cf[frameName].frames.rows[cooldownName..'_background'], optionsTable.row.background[pUnitType].color);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_backgroundTexture_editBox']:SetText(pTexturePath);
end


function CooldownFrames.row_setBackgroundColor(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.background[pUnitType].color.r;
	pG = pG or optionsTable.row.background[pUnitType].color.g;
	pB = pB or optionsTable.row.background[pUnitType].color.b;
	pA = pA or optionsTable.row.background[pUnitType].color.a;
	optionsTable.row.background[pUnitType].color.r = pR;
	optionsTable.row.background[pUnitType].color.g = pG;
	optionsTable.row.background[pUnitType].color.b = pB;
	optionsTable.row.background[pUnitType].color.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateRowBackgroundColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_backgroundColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


-- [COOLDOWN ICON] --


function CooldownFrames.row_setIconVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.icon[pUnitType].show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_iconShow_button']:SetChecked(pChecked);
	end
	optionsTable.row.icon[pUnitType].show = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
			CooldownFrames.updateRowItemPositions(frameName, pUnitType, cooldownName);
		end
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			CooldownFrames.updateStatusBarPositions(cf[frameName], cooldownName);
		end
	end
end


function CooldownFrames.updateRowItemPositions(pGroupName, pUnitType, pCooldownName)
	if ( cf[pGroupName].options.row.bars[pUnitType].mode == 'drain' ) then
		cf[pGroupName].frames.rows[pCooldownName..'_bar1']:ClearAllPoints();
		cf[pGroupName].frames.rows[pCooldownName..'_bar1']:SetPoint('RIGHT', cf[pGroupName].frames.rows[pCooldownName..'_background'], 'RIGHT');
		if ( not Spells.cooldownIndex[pCooldownName].ignoreInterval ) then
			cf[pGroupName].frames.rows[pCooldownName..'_bar2']:ClearAllPoints();
			cf[pGroupName].frames.rows[pCooldownName..'_bar2']:SetPoint('RIGHT', cf[pGroupName].frames.rows[pCooldownName..'_bar1'], 'LEFT');
			cf[pGroupName].frames.rows[pCooldownName..'_divider']:SetPoint('CENTER', cf[pGroupName].frames.rows[pCooldownName..'_bar1'], 'LEFT');
		end
		if ( cf[pGroupName].options.row.icon[pUnitType].show ) then
			cf[pGroupName].frames.rows[pCooldownName..'_icon']:Show();
			cf[pGroupName].frames.rows[pCooldownName..'_name']:SetPoint('LEFT', cf[pGroupName].frames.rows[pCooldownName..'_icon'], 'RIGHT', cf[pGroupName].options.row.textIndent, 0);
		else
			cf[pGroupName].frames.rows[pCooldownName..'_icon']:Hide();
			cf[pGroupName].frames.rows[pCooldownName..'_name']:SetPoint('LEFT', cf[pGroupName].frames.rows[pCooldownName..'_background'], 'LEFT', cf[pGroupName].options.row.textIndent, 0);
		end
	else
		cf[pGroupName].frames.rows[pCooldownName..'_bar1']:ClearAllPoints();
		if ( cf[pGroupName].options.row.icon[pUnitType].show ) then
			cf[pGroupName].frames.rows[pCooldownName..'_icon']:Show();
			cf[pGroupName].frames.rows[pCooldownName..'_bar1']:SetPoint('LEFT', cf[pGroupName].frames.rows[pCooldownName..'_icon'], 'RIGHT');
			cf[pGroupName].frames.rows[pCooldownName..'_name']:SetPoint('LEFT', cf[pGroupName].frames.rows[pCooldownName..'_icon'], 'RIGHT', cf[pGroupName].options.row.textIndent, 0);
		else
			cf[pGroupName].frames.rows[pCooldownName..'_icon']:Hide();
			cf[pGroupName].frames.rows[pCooldownName..'_bar1']:SetPoint('LEFT', cf[pGroupName].frames.rows[pCooldownName..'_background'], 'LEFT');
			cf[pGroupName].frames.rows[pCooldownName..'_name']:SetPoint('LEFT', cf[pGroupName].frames.rows[pCooldownName..'_background'], 'LEFT', cf[pGroupName].options.row.textIndent, 0);
		end
		if ( not Spells.cooldownIndex[pCooldownName].ignoreInterval ) then
			cf[pGroupName].frames.rows[pCooldownName..'_bar2']:ClearAllPoints();
			cf[pGroupName].frames.rows[pCooldownName..'_bar2']:SetPoint('LEFT', cf[pGroupName].frames.rows[pCooldownName..'_bar1'], 'RIGHT');
			cf[pGroupName].frames.rows[pCooldownName..'_divider']:SetPoint('CENTER', cf[pGroupName].frames.rows[pCooldownName..'_bar2'], 'LEFT');
		end
	end
end

function CooldownFrames.row_setIconLength(pLength, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pLength = tonumber(pLength);
	if ( pLength == nil or pLength <= 0 ) then
		pLength = optionsTable.row.icon[pUnitType].length;
	end
	optionsTable.row.icon[pUnitType].length = pLength;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
			cf[frameName].frames.rows[cooldownName..'_icon']:SetWidth(pLength);
			cf[frameName].frames.rows[cooldownName..'_icon']:SetHeight(pLength);
		end
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			CooldownFrames.updateStatusBarPositions(cf[frameName], cooldownName);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_iconLength_editBox']:SetText(pLength);
end


function CooldownFrames.row_setIconAlpha(pAlpha, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pAlpha = tonumber(pAlpha);
	if ( pAlpha == nil or pAlpha < 0 or pAlpha > 1 ) then
		pAlpha = optionsTable.row.icon[pUnitType].a;
	end
	optionsTable.row.icon[pUnitType].a = pAlpha
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
			cf[frameName].frames.rows[cooldownName..'_icon']:SetAlpha(pAlpha);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_iconAlpha_editBox']:SetText(pAlpha);
end


-- [COOLDOWN NAME] --


function CooldownFrames.row_setNameVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.name[pUnitType].show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_nameShow_button']:SetChecked(pChecked);
	end
	optionsTable.row.name[pUnitType].show = pChecked;
end


function CooldownFrames.row_setNameHighlightVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.name[pUnitType].showHighlight;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_nameShowHighlight_button']:SetChecked(pChecked);
	end
	optionsTable.row.name[pUnitType].showHighlight = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateRowNameColor(cf[frameName], cooldownName);
			end
		end
	end
end


function CooldownFrames.row_setNameColor(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.name[pUnitType].color.r;
	pG = pG or optionsTable.row.name[pUnitType].color.g;
	pB = pB or optionsTable.row.name[pUnitType].color.b;
	pA = pA or optionsTable.row.name[pUnitType].color.a;
	optionsTable.row.name[pUnitType].color.r = pR;
	optionsTable.row.name[pUnitType].color.g = pG;
	optionsTable.row.name[pUnitType].color.b = pB;
	optionsTable.row.name[pUnitType].color.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateRowNameColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_row_'..pUnitType..'_nameColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


function CooldownFrames.row_setNameHighlightColor(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.name[pUnitType].highlightColor.r;
	pG = pG or optionsTable.row.name[pUnitType].highlightColor.g;
	pB = pB or optionsTable.row.name[pUnitType].highlightColor.b;
	pA = pA or optionsTable.row.name[pUnitType].highlightColor.a;
	optionsTable.row.name[pUnitType].highlightColor.r = pR;
	optionsTable.row.name[pUnitType].highlightColor.g = pG;
	optionsTable.row.name[pUnitType].highlightColor.b = pB;
	optionsTable.row.name[pUnitType].highlightColor.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateRowNameColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_nameHighlightColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


function CooldownFrames.row_setNameIgnoreIntervalColor(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.name[pUnitType].ignoreIntervalColor.r;
	pG = pG or optionsTable.row.name[pUnitType].ignoreIntervalColor.g;
	pB = pB or optionsTable.row.name[pUnitType].ignoreIntervalColor.b;
	pA = pA or optionsTable.row.name[pUnitType].ignoreIntervalColor.a;
	optionsTable.row.name[pUnitType].ignoreIntervalColor.r = pR;
	optionsTable.row.name[pUnitType].ignoreIntervalColor.g = pG;
	optionsTable.row.name[pUnitType].ignoreIntervalColor.b = pB;
	optionsTable.row.name[pUnitType].ignoreIntervalColor.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateRowNameColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_nameIgnoreIntervalColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


function CooldownFrames.row_setNameFont(pFont, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pFont = pFont or optionsTable.row.name[pUnitType].font;
	optionsTable.row.name[pUnitType].font = pFont;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do	
			cf[frameName].frames.rows[cooldownName..'_name']:SetFont(pFont, optionsTable.row.name[pUnitType].fontSize);
			Tools.updateTextDimensions(cf[frameName].frames.rows[cooldownName..'_name'], nil, true);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_row_'..pUnitType..'_nameFont_editBox']:SetText(pFont);
end


function CooldownFrames.row_setNameFontSize(pFontSize, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pFontSize = tonumber(pFontSize);
	if ( pFontSize == nil or pFontSize <= 0 ) then
		pFontSize = optionsTable.row.name[pUnitType].fontSize;
	end
	optionsTable.row.name[pUnitType].fontSize = pFontSize;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do	
			cf[frameName].frames.rows[cooldownName..'_name']:SetFont(optionsTable.row.name[pUnitType].font, pFontSize);
			Tools.updateTextDimensions(cf[frameName].frames.rows[cooldownName..'_name'], nil, true);
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_row_'..pUnitType..'_nameFontSize_editBox']:SetText(pFontSize);
end


-- [COOLDOWN DURATION] --


function CooldownFrames.row_setCooldownVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.cooldown[pUnitType].show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_cooldownShow_button']:SetChecked(pChecked);
	end
	optionsTable.row.cooldown[pUnitType].show = pChecked;
end


function CooldownFrames.row_setCooldownShowInMinutes(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.cooldown[pUnitType].showInMinutes;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_cooldownShowInMinutes_button']:SetChecked(pChecked);
	end
	optionsTable.row.cooldown[pUnitType].showInMinutes = pChecked;
end


function CooldownFrames.row_setCooldownEnableCutoff(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.cooldown[pUnitType].enableCutoff;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_cooldownEnableCutoff_button']:SetChecked(pChecked);
	end
	optionsTable.row.cooldown[pUnitType].enableCutoff = pChecked;
end


function CooldownFrames.row_setCooldownHighlightColorVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.cooldown[pUnitType].showHighlight;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_cooldownShowHightlight_button']:SetChecked(pChecked);
	end
	optionsTable.row.cooldown[pUnitType].showHighlight = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateRowCountdownColor(cf[frameName], cooldownName);
			end
		end
	end
end


function CooldownFrames.row_setCooldownCountdownBothBars(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.cooldown[pUnitType].countdownBothBars;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_cooldownCountdownBothBars_button']:SetChecked(pChecked);
	end
	optionsTable.row.cooldown[pUnitType].countdownBothBars = pChecked;
end


function CooldownFrames.row_setCooldownColor(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.cooldown[pUnitType].color.r;
	pG = pG or optionsTable.row.cooldown[pUnitType].color.g;
	pB = pB or optionsTable.row.cooldown[pUnitType].color.b;
	pA = pA or optionsTable.row.cooldown[pUnitType].color.a;
	optionsTable.row.cooldown[pUnitType].color.r = pR;
	optionsTable.row.cooldown[pUnitType].color.g = pG;
	optionsTable.row.cooldown[pUnitType].color.b = pB;
	optionsTable.row.cooldown[pUnitType].color.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateRowCountdownColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_row_'..pUnitType..'_cooldownColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


function CooldownFrames.row_setCooldownHighlightColor(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.cooldown[pUnitType].highlightColor.r;
	pG = pG or optionsTable.row.cooldown[pUnitType].highlightColor.g;
	pB = pB or optionsTable.row.cooldown[pUnitType].highlightColor.b;
	pA = pA or optionsTable.row.cooldown[pUnitType].highlightColor.a;
	optionsTable.row.cooldown[pUnitType].highlightColor.r = pR;
	optionsTable.row.cooldown[pUnitType].highlightColor.g = pG;
	optionsTable.row.cooldown[pUnitType].highlightColor.b = pB;
	optionsTable.row.cooldown[pUnitType].highlightColor.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateRowCountdownColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_cooldownHighlightColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


function CooldownFrames.row_setCooldownIgnoreIntervalColor(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.cooldown[pUnitType].ignoreIntervalColor.r;
	pG = pG or optionsTable.row.cooldown[pUnitType].ignoreIntervalColor.g;
	pB = pB or optionsTable.row.cooldown[pUnitType].ignoreIntervalColor.b;
	pA = pA or optionsTable.row.cooldown[pUnitType].ignoreIntervalColor.a;
	optionsTable.row.cooldown[pUnitType].ignoreIntervalColor.r = pR;
	optionsTable.row.cooldown[pUnitType].ignoreIntervalColor.g = pG;
	optionsTable.row.cooldown[pUnitType].ignoreIntervalColor.b = pB;
	optionsTable.row.cooldown[pUnitType].ignoreIntervalColor.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateRowCountdownColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_cooldownIgnoreIntervalColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


function CooldownFrames.row_setCooldownFont(pFont, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pFont = pFont or optionsTable.row.cooldown[pUnitType].font;
	optionsTable.row.cooldown[pUnitType].font = pFont;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do	
			cf[frameName].frames.rows[cooldownName..'_cooldown']:SetFont(pFont, optionsTable.row.cooldown[pUnitType].fontSize);
		end
		
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_row_'..pUnitType..'_cooldownFont_editBox']:SetText(pFont);
end


function CooldownFrames.row_setCooldownFontSize(pFontSize, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pFontSize = tonumber(pFontSize);
	if ( pFontSize == nil or pFontSize <= 0 ) then
		pFontSize = optionsTable.row.cooldown[pUnitType].fontSize;
	end
	optionsTable.row.cooldown[pUnitType].fontSize = pFontSize;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
			cf[frameName].frames.rows[cooldownName..'_cooldown']:SetFont(optionsTable.row.cooldown[pUnitType].font, pFontSize);
		end
		
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_font_row_'..pUnitType..'_cooldownFontSize_editBox']:SetText(pFontSize);
end


function CooldownFrames.row_setCooldownCutoff(pCutoff, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pCutoff = tonumber(pCutoff);
	if ( pCutoff == nil or pCutoff <= 0 ) then
		pCutoff = optionsTable.row.cooldown[pUnitType].cutoffThreshold / 1000;
	else
		optionsTable.row.cooldown[pUnitType].cutoffThreshold = pCutoff * 1000;
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_cooldownCutoff_editBox']:SetText(Tools.roundNumber(pCutoff));
end


-- [BARS] --


function CooldownFrames.row_setBarsVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.bars[pUnitType].show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_barsShow_button']:SetChecked(pChecked);
	end
	optionsTable.row.bars[pUnitType].show = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			CooldownFrames.updateStatusBarPositions(cf[frameName], cooldownName);
		end
	end
end


function CooldownFrames.row_setBarsMode(pPoint, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pPoint = pPoint or optionsTable.row.bars[pUnitType].mode;
	optionsTable.row.bars[pUnitType].mode = pPoint;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
			CooldownFrames.updateRowItemPositions(frameName, pUnitType, cooldownName);
		end
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			CooldownFrames.updateStatusBarPositions(cf[frameName], cooldownName);
			cf[frameName].tables.cooldownStateIndex[cooldownName] = 'primary';
		end
	end
	-- Update Options Frame
	if ( pPoint == 'drain' ) then
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_barsModeFill_button']:SetChecked(false);
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_barsModeDrain_button']:SetChecked(true);
	else
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_barsModeFill_button']:SetChecked(true);
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_barsModeDrain_button']:SetChecked(false);
	end
end


-- [BAR1] --


function CooldownFrames.row_setBar1TextureVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.bar1[pUnitType].showTexture;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_bar1EnableTexture_button']:SetChecked(pChecked);
	end
	optionsTable.row.bar1[pUnitType].showTexture = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( pChecked ) then
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( not cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_bar1']:SetStatusBarTexture(optionsTable.row.bar1[pUnitType].texture);
				end
			end
		else
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( not cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_bar1']:SetStatusBarTexture([[interface\addons\woogled\textures\blank.tga]]);
				end
			end
		end
	end
end


function CooldownFrames.row_setBar1Texture(pTexturePath, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pTexturePath = pTexturePath or optionsTable.row.bar1[pUnitType].texture;
	optionsTable.row.bar1[pUnitType].texture = pTexturePath;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( optionsTable.row.bar1[pUnitType].showTexture ) then
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( not cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_bar1']:SetStatusBarTexture(pTexturePath);
				end
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_bar1Texture_editBox']:SetText(pTexturePath);
end


function CooldownFrames.row_setBar1Color(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.bar1[pUnitType].color.r;
	pG = pG or optionsTable.row.bar1[pUnitType].color.g;
	pB = pB or optionsTable.row.bar1[pUnitType].color.b;
	pA = pA or optionsTable.row.bar1[pUnitType].color.a;
	optionsTable.row.bar1[pUnitType].color.r = pR;
	optionsTable.row.bar1[pUnitType].color.g = pG;
	optionsTable.row.bar1[pUnitType].color.b = pB;
	optionsTable.row.bar1[pUnitType].color.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updatePrimaryBarColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_bar1Color_button'].texture:SetTexture(pR, pG, pB, pA);
end


-- [BAR2] --


function CooldownFrames.row_setBar2TextureVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.bar2[pUnitType].showTexture;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_bar2EnableTexture_button']:SetChecked(pChecked);
	end
	optionsTable.row.bar2[pUnitType].showTexture = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( pChecked ) then
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( not cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_bar2']:SetStatusBarTexture(optionsTable.row.bar2[pUnitType].texture);
				end
			end
		else
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( not cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_bar2']:SetStatusBarTexture([[interface\addons\woogled\textures\blank.tga]]);
				end
			end
		end
	end
end


function CooldownFrames.row_setBar2Texture(pTexturePath, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pTexturePath = pTexturePath or optionsTable.row.bar2[pUnitType].texture;
	optionsTable.row.bar2[pUnitType].texture = pTexturePath;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( optionsTable.row.bar2[pUnitType].showTexture ) then
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( not cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_bar2']:SetStatusBarTexture(pTexturePath);
				end
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_bar2Texture_editBox']:SetText(pTexturePath);
end


function CooldownFrames.row_setBar2Color(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.bar2[pUnitType].color.r;
	pG = pG or optionsTable.row.bar2[pUnitType].color.g;
	pB = pB or optionsTable.row.bar2[pUnitType].color.b;
	pA = pA or optionsTable.row.bar2[pUnitType].color.a;
	optionsTable.row.bar2[pUnitType].color.r = pR;
	optionsTable.row.bar2[pUnitType].color.g = pG;
	optionsTable.row.bar2[pUnitType].color.b = pB;
	optionsTable.row.bar2[pUnitType].color.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateSecondaryBarColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_bar2Color_button'].texture:SetTexture(pR, pG, pB, pA);
end


-- [DIVIDER] --


function CooldownFrames.row_setDividerVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.divider[pUnitType].show;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_dividerShow_button']:SetChecked(pChecked);
	end
	optionsTable.row.divider[pUnitType].show = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			CooldownFrames.updateStatusBarPositions(cf[frameName], cooldownName);
		end
	end
end


function CooldownFrames.row_setDividerTextureVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.divider[pUnitType].showTexture;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_dividerEnableTexture_button']:SetChecked(pChecked);
	end
	optionsTable.row.divider[pUnitType].showTexture = false;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( pChecked ) then
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( not cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_divider'].texture:SetTexture(nil);
					cf[frameName].frames.rows[cooldownName..'_divider']:SetBackdrop({bgFile = optionsTable.row.divider[pUnitType].texture});
					Tools.setBackdropColor(cf[frameName].frames.rows[cooldownName..'_divider'], optionsTable.row.divider[pUnitType].color);
				end
			end
		else
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( not cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_divider']:SetBackdrop(nil);
					cf[frameName].frames.rows[cooldownName..'_divider']:SetBackdropColor(nil);
					Tools.setColor(cf[frameName].frames.rows[cooldownName..'_divider'].texture, optionsTable.row.divider[pUnitType].color);
				end
			end
		end
	end
end


function CooldownFrames.row_setDividerWidth(pWidth, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pWidth = tonumber(pWidth);
	if ( pWidth == nil or pWidth <= 0 ) then
		pWidth = optionsTable.row.divider[pUnitType].width;
	end
	optionsTable.row.divider[pUnitType].width = pWidth;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
			if ( not cooldownTable.ignoreInterval ) then
				cf[frameName].frames.rows[cooldownName..'_divider']:SetWidth(pWidth);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_dividerWidth_editBox']:SetText(pWidth);
end


function CooldownFrames.row_setDividerTexture(pTexturePath, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pTexturePath = pTexturePath or optionsTable.row.divider[pUnitType].texture;
	optionsTable.row.divider[pUnitType].texture = pTexturePath;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( optionsTable.row.divider[pUnitType].showTexture ) then
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( not cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_divider']:SetBackdrop({bgFile = pTexturePath});
					Tools.setBackdropColor(cf[frameName].frames.rows[cooldownName..'_divider'], optionsTable.row.divider[pUnitType].color);
				end
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_dividerTexture_editBox']:SetText(pTexturePath);
end


function CooldownFrames.row_setDividerColor(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.divider[pUnitType].color.r;
	pG = pG or optionsTable.row.divider[pUnitType].color.g;
	pB = pB or optionsTable.row.divider[pUnitType].color.b;
	pA = pA or optionsTable.row.divider[pUnitType].color.a;
	optionsTable.row.divider[pUnitType].color.r = pR;
	optionsTable.row.divider[pUnitType].color.g = pG;
	optionsTable.row.divider[pUnitType].color.b = pB;
	optionsTable.row.divider[pUnitType].color.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updateDividerColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_dividerColor_button'].texture:SetTexture(pR, pG, pB, pA);
end


-- [IGNORE INTERVAL] --


function CooldownFrames.row_setIgnoreIntervalEnable(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.ignoreInterval[pUnitType].enable;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_row_'..pUnitType..'_ignoreIntervalEnable_button']:SetChecked(pChecked);
	end
	optionsTable.row.ignoreInterval[pUnitType].enable = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			CooldownFrames.updatePrimaryBarColor(cf[frameName], cooldownName);
			CooldownFrames.updateRowNameColor(cf[frameName], cooldownName)
			CooldownFrames.updateRowCountdownColor(cf[frameName], cooldownName)
		end
	end
end


function CooldownFrames.row_setIgnoreIntervalTextureVisibility(pChecked, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	if ( pChecked == nil ) then
		pChecked = optionsTable.row.ignoreInterval[pUnitType].showTexture;
		-- Update Options Frame
		OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_ignoreIntervalEnableTexture_button']:SetChecked(pChecked);
	end
	optionsTable.row.ignoreInterval[pUnitType].showTexture = pChecked;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( pChecked ) then
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_bar1']:SetStatusBarTexture(optionsTable.row.ignoreInterval[pUnitType].texture);
				end
			end
		else
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_bar1']:SetStatusBarTexture([[interface\addons\woogled\textures\blank.tga]]);
				end
			end
		end
	end
end


function CooldownFrames.row_setIgnoreIntervalTexture(pTexturePath, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pTexturePath = pTexturePath or optionsTable.row.ignoreInterval[pUnitType].texture;
	optionsTable.row.ignoreInterval[pUnitType].texture = pTexturePath;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		if ( optionsTable.row.ignoreInterval[pUnitType].showTexture ) then
			for cooldownName, cooldownTable in pairs ( Spells.cooldownIndexByUnitType[pUnitType] ) do
				if ( cooldownTable.ignoreInterval ) then
					cf[frameName].frames.rows[cooldownName..'_bar1']:SetStatusBarTexture(pTexturePath);
				end
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_ignoreIntervalTexture_editBox']:SetText(pTexturePath);
end


function CooldownFrames.row_setIgnoreIntervalColor(pR, pG, pB, pA, pOptionsName, pUnitType)
	local frameNameArray = groupsTableIndexByGroupName[pOptionsName];
	local optionsTable = cf[frameNameArray[1]].options;
	-- Update options
	pR = pR or optionsTable.row.ignoreInterval[pUnitType].color.r;
	pG = pG or optionsTable.row.ignoreInterval[pUnitType].color.g;
	pB = pB or optionsTable.row.ignoreInterval[pUnitType].color.b;
	pA = pA or optionsTable.row.ignoreInterval[pUnitType].color.a;
	optionsTable.row.ignoreInterval[pUnitType].color.r = pR;
	optionsTable.row.ignoreInterval[pUnitType].color.g = pG;
	optionsTable.row.ignoreInterval[pUnitType].color.b = pB;
	optionsTable.row.ignoreInterval[pUnitType].color.a = pA;
	-- Update Cooldown Frames
	for i, frameName in pairs ( frameNameArray ) do
		for i, cooldownName in pairs ( cf[frameName].tables.cooldownNameArray ) do
			if ( Spells.unitTypeIndexByCooldownName[cooldownName] == pUnitType ) then
				CooldownFrames.updatePrimaryBarColor(cf[frameName], cooldownName);
			end
		end
	end
	-- Update Options Frame
	OptionsFrame.groupFrames[pOptionsName..'_texture_row_'..pUnitType..'_ignoreIntervalColor_button'].texture:SetTexture(pR, pG, pB, pA);
end