----------
-- Name: WOOGLED
-- Description: A player and pet cooldown-tracking addon for WoW
-- Author: Woogs - Greymane (US)
-- Version: 2.0.x
-- License: All Rights Reserved
----------


--=====--=====--=====--
-- [PRIVATE PROPERTIES]
--=====--=====--=====--


WOOGLED.MinimapFrame.frames = {};
WOOGLED.MinimapFrame.buttons = {};
WOOGLED.MinimapFrame.text = {};


local Libs = WOOGLED.Libs;
local Spells = WOOGLED.Spells;
local Tools = WOOGLED.Tools;
local Units = WOOGLED.Units;
local PlayerFrame = WOOGLED.PlayerFrame;
local CooldownFrames = WOOGLED.CooldownFrames;
local FocusFrame = WOOGLED.FocusFrame;
local OptionsFrame = WOOGLED.OptionsFrame;
local MinimapFrame = WOOGLED.MinimapFrame;

local toggles = WOOGLED.toggles;

local frames = WOOGLED.MinimapFrame.frames;
local buttons = WOOGLED.MinimapFrame.buttons;
local text = WOOGLED.MinimapFrame.text;


--=====--=====--=====--
-- [PRIVATE METHODS]
--=====--=====--=====--


function MinimapFrame.build()
	frames.anchor = CreateFrame('Button', 'WOOGLED_MINIMAPFRAME_FRAMES_ANCHOR', Minimap);
	frames.anchor:SetWidth(32);
	frames.anchor:SetHeight(32);
	frames.anchor:EnableMouse(true);
	frames.anchor:SetMovable(true);
	frames.anchor:SetFrameStrata('LOW');
	frames.anchor:SetScript('OnDragStart', MinimapFrame.onDragStart);
	frames.anchor:SetScript('OnDragStop', MinimapFrame.onDragStop);
	frames.anchor:SetScript('OnClick', function() WOOGLED.setParse(not WOOGLED.parse) end);
	local radius = frames.anchor:GetWidth() / 2 + 63;
	frames.anchor:SetPoint('CENTER', Minimap, 'CENTER', 0.0 - (radius * cos(WOOGLED_OPTIONS.mf.angle)), (radius * sin(WOOGLED_OPTIONS.mf.angle)) - 1.0)
	frames.anchor:RegisterForClicks('LeftButtonUp');
	frames.anchor:RegisterForDrag('LeftButton');
	
	frames.anchorBgTexture = frames.anchor:CreateTexture('WOOGLED_MINIMAPFRAME_FRAMES_ANCHORBGTEXTURE', 'BACKGROUND');
	frames.anchorBgTexture:SetTexture('Interface\\Icons\\Inv_misc_summerfest_brazierred');
	frames.anchorBgTexture:SetWidth(19);
	frames.anchorBgTexture:SetHeight(19);
	frames.anchorBgTexture:SetPoint('CENTER', frames.anchor, 'CENTER', -0.5, 1.5); 
	
	frames.anchorBorderTexture = frames.anchor:CreateTexture('WOOGLED_MINIMAPFRAME_FRAMES_ANCHORBORDERTEXTURE', 'OVERLAY');
	frames.anchorBorderTexture:SetWidth(52);
	frames.anchorBorderTexture:SetHeight(52);
	frames.anchorBorderTexture:SetTexture('Interface\\Minimap\\MiniMap-TrackingBorder');
	frames.anchorBorderTexture:SetPoint('TOPLEFT', frames.anchor, 'TOPLEFT');
	
	frames.anchor.texture = frames.anchorBgTexture;
	frames.anchor.texture = frames.anchorBorderTexture;
end


function MinimapFrame.onDragStart()
	frames.anchor:SetScript('OnUpdate', MinimapFrame.move);
end


function MinimapFrame.onDragStop()
	frames.anchor:SetScript('OnUpdate', nil)
end


function MinimapFrame.getAngle()
	local mouseX, mouseY = GetCursorPosition();
	local mapX, mapY = Minimap:GetLeft(), Minimap:GetBottom();

	mouseX = (mapX - (mouseX / UIParent:GetScale())) + 70.5;
	mouseY = ((mouseY / UIParent:GetScale()) - mapY) - 70.5;
	
	local angle = math.deg(math.atan2(mouseY, mouseX));
	
	return angle;
end


function MinimapFrame.move()
	local angle = MinimapFrame.getAngle();
	local radius = frames.anchor:GetWidth() / 2 + 63;
	frames.anchor:SetPoint('CENTER', Minimap, 'CENTER', 0.0 - (radius * cos(angle)), (radius * sin(angle)) - 1.0);
	WOOGLED_OPTIONS.mf.angle = angle;
end