﻿----------
-- Name: WOOGLED
-- Description: A player and pet cooldown-tracking addon for WoW
-- Author: Woogs - Greymane (US)
-- Version: 2.0.x
-- License: All Rights Reserved
----------


WOOGLED = {
	version = '2.0.0-beta16',
	parse = false,
	player = {
	},
	time = {
		now = 0,
		frames = {
			last = 0,
			cooldown = 100,
		},
		deleteInvalidUnits = {
			last = 0,
			cooldown = 10000,
		},
		woogsCheck = {
			last = 0,
			cooldown = 5000,
			counter = 0,
			found = 0,
			state = false,
		},
	},
	Zones = {},
	Libs = {},
	Spells = {},
	Tools = {},
	Units = {},
	CooldownFrames = {},
	OptionsFrame = {},
	MinimapFrame = {},
};


local WOOGLED = WOOGLED;
local Zones = WOOGLED.Zones;
local Libs = WOOGLED.Libs;
local Spells = WOOGLED.Spells;
local Tools = WOOGLED.Tools;
local Units = WOOGLED.Units;
local CooldownFrames = WOOGLED.CooldownFrames;
local OptionsFrame = WOOGLED.OptionsFrame;
local MinimapFrame = WOOGLED.MinimapFrame;


WOOGLED.eventHandler = CreateFrame('Frame', 'WOOGLED_EVENTHANDLER');
WOOGLED.eventHandler:RegisterEvent('ADDON_LOADED');
WOOGLED.eventHandler:RegisterEvent('PLAYER_ENTERING_WORLD');
WOOGLED.eventHandler:RegisterEvent('PLAYER_LOGIN');
WOOGLED.eventHandler:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED');
WOOGLED.eventHandler:RegisterEvent('ZONE_CHANGED_NEW_AREA');
WOOGLED.eventHandler:RegisterEvent('PLAYER_TARGET_CHANGED');
WOOGLED.eventHandler:RegisterEvent('PLAYER_FOCUS_CHANGED');
WOOGLED.eventHandler:RegisterEvent('CHAT_MSG_SYSTEM');
WOOGLED.eventHandler:RegisterEvent('DUEL_FINISHED');
WOOGLED.eventHandler:SetScript('OnUpdate', function(self, elapsed) WOOGLED.onUpdate(self, elapsed); end);
WOOGLED.eventHandler:SetScript('OnEvent', function(self, eventType, ...) WOOGLED.onEvent(self, eventType, ...); end);


function WOOGLED.onUpdate(pSelf, pElapsed)
	pElapsed = Tools.roundNumber(pElapsed * 1000);
	WOOGLED.time.now = WOOGLED.time.now + pElapsed;
	WOOGLED.time.frames.last = WOOGLED.time.frames.last + pElapsed;
	while ( WOOGLED.time.frames.last >= WOOGLED.time.frames.cooldown ) do
		CooldownFrames.update();
		WOOGLED.time.frames.last = WOOGLED.time.frames.last - WOOGLED.time.frames.cooldown;
	end
	WOOGLED.time.deleteInvalidUnits.last = WOOGLED.time.deleteInvalidUnits.last + pElapsed;
	while ( WOOGLED.time.deleteInvalidUnits.last >= WOOGLED.time.deleteInvalidUnits.cooldown ) do
		Units.deleteInvalidUnits();
		WOOGLED.time.deleteInvalidUnits.last = WOOGLED.time.deleteInvalidUnits.last - WOOGLED.time.deleteInvalidUnits.cooldown;
	end
	if ( WOOGLED.time.woogsCheck.state ) then
		WOOGLED.time.woogsCheck.last = WOOGLED.time.woogsCheck.last + pElapsed;
		while ( WOOGLED.time.woogsCheck.last >= WOOGLED.time.woogsCheck.cooldown ) do
			WOOGLED.woogsCheck();
			WOOGLED.time.woogsCheck.last = WOOGLED.time.woogsCheck.last - WOOGLED.time.woogsCheck.cooldown;
		end
	end
end


function WOOGLED.onEvent(pSelf, pEventType, ...)
	if ( pEventType == 'ADDON_LOADED' and select(1, ...) == 'WOOGLED' ) then
		WOOGLED.player.locale = GetLocale();
		WOOGLED.buildDefaultOptions();
		WOOGLED.checkSavedVariables();
		Spells.buildCooldownIndex();
		OptionsFrame.build();
		CooldownFrames.build();
		MinimapFrame.build();
		SLASH_WOOGLED1 = '/woogled';
--		WOOGLED.LibSharedMedia = LibStub("LibSharedMedia-3.0");
	--	WOOGLED.LibSharedMedia:Register('statusbar', 'WOOGLED Blank', [[interface\addons\woogled\textures\blank.tga]]);
	elseif ( pEventType == 'PLAYER_ENTERING_WORLD' ) then
		Zones.onZoneChange();
	elseif ( pEventType == 'PLAYER_LOGIN' ) then
		WOOGLED.player.guid = UnitGUID('player');
		WOOGLED.player.name = UnitName('player');
		WOOGLED.player.faction = UnitFactionGroup('player');
		WOOGLED.player.realm = GetRealmName();
	elseif ( pEventType == 'COMBAT_LOG_EVENT_UNFILTERED' and WOOGLED.parse ) then
		Units.processEvent(select(1, ...)*1000, select(2, ...), select(4, ...), select(5, ...), select(8, ...), select(9, ...), select(12, ...));
	elseif ( pEventType == 'ZONE_CHANGED_NEW_AREA' ) then
		Zones.onZoneChange();
	elseif ( pEventType == 'PLAYER_TARGET_CHANGED' ) then
		if ( Zones.zoneOptionsTable.enableTF ) then
			CooldownFrames.onTargetChange();
		end
	elseif ( pEventType == 'PLAYER_FOCUS_CHANGED' ) then
		if ( Zones.zoneOptionsTable.enableFF ) then
			CooldownFrames.onFocusChange();
		end
	elseif ( pEventType == 'CHAT_MSG_SYSTEM' ) then
		if ( WOOGLED_OPTIONS.zones.duel.enableTF or WOOGLED_OPTIONS.zones.duel.enableFF ) then
			local message = select(1, ...);
			if ( message == Libs.duel.trigger[WOOGLED.player.locale] ) then
				if ( WOOGLED_OPTIONS.zones.duel.enableTF ) then
					CooldownFrames.setVisibility(true, 'tf');
				end
				if ( WOOGLED_OPTIONS.zones.duel.enableFF ) then
					CooldownFrames.setVisibility(true, 'ff');
				end
				WOOGLED.setParse(true);
			end
		end
	elseif ( pEventType == 'DUEL_FINISHED' ) then
		Zones.setZoneOptions();
	end
end


function WOOGLED.checkSavedVariables()
	if ( WOOGLED_OPTIONS == nil ) then
		WOOGLED_OPTIONS = {};
		Tools.compareTables('copy', WOOGLED.defaultOptions, WOOGLED_OPTIONS);
		return;
	end
	if ( WOOGLED_OPTIONS.ignoreSpells == nil ) then WOOGLED_OPTIONS.ignoreSpells = {}; end
	for k, v in pairs ( WOOGLED.defaultOptions ) do
		if ( type(v) == 'table' ) then
			if ( k ~= 'ignoreSpells' ) then
				if ( WOOGLED_OPTIONS[k] == nil or type(WOOGLED_OPTIONS[k]) ~= 'table' ) then WOOGLED_OPTIONS[k] = {}; end
				Tools.compareTables('remove', WOOGLED.defaultOptions[k], WOOGLED_OPTIONS[k]);
				Tools.compareTables('add', WOOGLED.defaultOptions[k], WOOGLED_OPTIONS[k]);
			end
		else
			WOOGLED_OPTIONS[k] = nil;
		end
	end
	Tools.compareTables('copy', WOOGLED.defaultOptions.of, WOOGLED_OPTIONS.of);
end


function WOOGLED.buildDefaultOptions()
	WOOGLED.defaultOptions = {
		parse = {
			self = true,
			insideRaid = true,
			outsideRaid = true,
			pet = true,
		},

-- [TARGET FRAME] --

		tf = {
			scale = 1,
			lock = false,
			stickyTargeting = true,
			showPet = true,
			endpoint = {
				endPoint1 = 'shortest', -- shortest / interval / longest
				endPoint2 = 'interval', -- disable / interval / longest
			},
			anchor = {
				background = {
					width = 175,
					height = 36,
					r = 0,
					g = 0,
					b = 0,
					a = 1,
					showTexture = false,
					texture = [[interface\addons\woogled\textures\blank.tga]],
				},
				class = {
					show = true,
					length = 18,
					a = 1,
				},
				spec = {
					show = true,
					length = 16,
					a = 1,
				},
				pet = {
					show = true,
					length = 16,
					a = 1,
				},
				pin = {
					show = true,
					length = 18,
					a = 1,
				},
				name = {
					show = true,					
					r = 1,
					g = 1,
					b = 1,
					a = 1,
					font = [[Fonts\ZYKai_T.TTF]],
					fontSize = 12,
				},
				server = {
					show = true,
					r = 0,
					g = 1,
					b = 0.86,
					a = 1,
					font = [[Fonts\ZYKai_T.TTF]],
					fontSize = 12,
				},
			},
			row = {
				numRows = 15,
				textIndent = 3,
				sortBy = 'cooldown', -- event / cooldown
				alignment = 'center', -- left / center / right
				buildDirection = 'down', -- up / down
				stackDirection = 'down', -- up / down
				background = {
					spacing = 0,
					player = {
						width = 175,
						height = 16,
						color = {r = 0, g = 0, b = 0, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						width = 175,
						height = 16,
						color = {r = 0, g = 0, b = 0, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				bars = {
					player = {
						show = true,
						mode = 'fill', -- drain / fill
					},
					pet = {
						show = true,
						mode = 'fill', -- drain / fill
					},
				},
				bar1 = {
					player = {
						color = {r = 0.4, g = 0.4, b = 0.4, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						color = {r = 0.2, g = 0.2, b = 0.2, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				bar2 = {
					player = {
						color = {r = 0, g = 0.6, b = 0.8, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						color = {r = 0, g = 0.7, b = 0.4, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				divider = {
					player = {
						show = false,
						width = 2,
						color = {r = 0, g = 0.4, b = 0.6, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						show = false,
						width = 2,
						color = {r = 0, g = 0.6, b = 0.3, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				ignoreInterval = {
					player = {
						enable = true,
						color = {r = 0, g = 0.6, b = 0.8, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						enable = true,
						color = {r = 0, g = 0.7, b = 0.4, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},				
				icon = {
					player = {
						show = true,
						length = 15,
						a = 1,
					},
					pet = {
						show = true,
						length = 15,
						a = 1,
					},
				},				
				name = {
					player = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},							
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showHighlight = true,
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
					},
					pet = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showHighlight = true,
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
					},
				},
				cooldown = {
					player = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showInMinutes = true,
						showHighlight = true,				
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
						countdownBothBars = true,
						enableCutoff = false,
						cutoffThreshold = 15000,
					},
					pet = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showInMinutes = true,
						showHighlight = true,			
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
						countdownBothBars = true,
						enableCutoff = false,
						cutoffThreshold = 15000,
					},
				},
			},
		},
		
-- [FOCUS FRAME] --

		ff = {
			scale = 1,
			lock = false,
			showPet = true,
			endpoint = {
				endPoint1 = 'shortest', -- shortest / interval / longest
				endPoint2 = 'interval', -- disable / interval / longest
			},
			anchor = {
				background = {
					width = 175,
					height = 36,
					r = 0,
					g = 0,
					b = 0,
					a = 1,
					showTexture = false,
					texture = [[interface\addons\woogled\textures\blank.tga]],
				},
				class = {
					show = true,
					length = 18,
					a = 1,
				},
				spec = {
					show = true,
					length = 16,
					a = 1,
				},
				pet = {
					show = true,
					length = 16,
					a = 1,
				},
				name = {
					show = true,					
					r = 1,
					g = 1,
					b = 1,
					a = 1,
					font = [[Fonts\ZYKai_T.TTF]],
					fontSize = 12,
				},
				server = {
					show = true,
					r = 0,
					g = 1,
					b = 0.86,
					a = 1,
					font = [[Fonts\ZYKai_T.TTF]],
					fontSize = 12,
				},
			},
			row = {
				numRows = 15,
				textIndent = 3,
				sortBy = 'cooldown', -- event / cooldown
				alignment = 'center', -- left / center / right
				buildDirection = 'down', -- up / down
				stackDirection = 'down', -- up / down
				background = {
					spacing = 0,
					player = {
						width = 175,
						height = 16,
						color = {r = 0, g = 0, b = 0, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						width = 175,
						height = 16,
						color = {r = 0, g = 0, b = 0, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				bars = {
					player = {
						show = true,
						mode = 'fill', -- drain / fill
					},
					pet = {
						show = true,
						mode = 'fill', -- drain / fill
					},
				},
				bar1 = {
					player = {
						color = {r = 0.4, g = 0.4, b = 0.4, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						color = {r = 0.2, g = 0.2, b = 0.2, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				bar2 = {
					player = {
						color = {r = 0, g = 0.6, b = 0.8, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						color = {r = 0, g = 0.7, b = 0.4, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				divider = {
					player = {
						show = false,
						width = 2,
						color = {r = 0, g = 0.4, b = 0.6, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						show = false,
						width = 2,
						color = {r = 0, g = 0.6, b = 0.3, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				ignoreInterval = {
					player = {
						enable = true,
						color = {r = 0, g = 0.6, b = 0.8, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						enable = true,
						color = {r = 0, g = 0.7, b = 0.4, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},				
				icon = {
					player = {
						show = true,
						length = 15,
						a = 1,
					},
					pet = {
						show = true,
						length = 15,
						a = 1,
					},
				},				
				name = {
					player = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},							
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showHighlight = true,
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
					},
					pet = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showHighlight = true,
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
					},
				},
				cooldown = {
					player = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showInMinutes = true,
						showHighlight = true,				
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
						countdownBothBars = true,
						enableCutoff = false,
						cutoffThreshold = 15000,
					},
					pet = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showInMinutes = true,
						showHighlight = true,			
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
						countdownBothBars = true,
						enableCutoff = false,
						cutoffThreshold = 15000,
					},
				},
			},
		},
		
-- [EXTRA FRAMES] --

		ef = {
			scale = 1,
			lock = false,
			showPet = true,
			endpoint = {
				endPoint1 = 'shortest', -- shortest / interval / longest
				endPoint2 = 'interval', -- disable / interval / longest
			},
			anchor = {
				background = {
					width = 175,
					height = 36,
					r = 0,
					g = 0,
					b = 0,
					a = 1,
					showTexture = false,
					texture = [[interface\addons\woogled\textures\blank.tga]],
				},
				class = {
					show = true,
					length = 18,
					a = 1,
				},
				spec = {
					show = true,
					length = 16,
					a = 1,
				},
				pet = {
					show = true,
					length = 16,
					a = 1,
				},
				close = {
					length = 18,
					a = 1,
				},
				name = {
					show = true,					
					r = 1,
					g = 1,
					b = 1,
					a = 1,
					font = [[Fonts\ZYKai_T.TTF]],
					fontSize = 12,
				},
				server = {
					show = true,
					r = 0,
					g = 1,
					b = 0.86,
					a = 1,
					font = [[Fonts\ZYKai_T.TTF]],
					fontSize = 12,
				},
			},
			row = {
				numRows = 15,
				textIndent = 3,
				sortBy = 'cooldown', -- event / cooldown
				alignment = 'center', -- left / center / right
				buildDirection = 'down', -- up / down
				stackDirection = 'down', -- up / down
				background = {
					spacing = 0,
					player = {
						width = 175,
						height = 16,
						color = {r = 0, g = 0, b = 0, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						width = 175,
						height = 16,
						color = {r = 0, g = 0, b = 0, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				bars = {
					player = {
						show = true,
						mode = 'fill', -- drain / fill
					},
					pet = {
						show = true,
						mode = 'fill', -- drain / fill
					},
				},
				bar1 = {
					player = {
						color = {r = 0.4, g = 0.4, b = 0.4, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						color = {r = 0.2, g = 0.2, b = 0.2, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				bar2 = {
					player = {
						color = {r = 0, g = 0.6, b = 0.8, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						color = {r = 0, g = 0.7, b = 0.4, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				divider = {
					player = {
						show = false,
						width = 2,
						color = {r = 0, g = 0.4, b = 0.6, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						show = false,
						width = 2,
						color = {r = 0, g = 0.6, b = 0.3, a = 1},
						showTexture = false,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},
				ignoreInterval = {
					player = {
						enable = true,
						color = {r = 0, g = 0.6, b = 0.8, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
					pet = {
						enable = true,
						color = {r = 0, g = 0.7, b = 0.4, a = 1},
						showTexture = true,
						texture = [[interface\addons\woogled\textures\blank.tga]],
					},
				},				
				icon = {
					player = {
						show = true,
						length = 15,
						a = 1,
					},
					pet = {
						show = true,
						length = 15,
						a = 1,
					},
				},				
				name = {
					player = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},							
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showHighlight = true,
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
					},
					pet = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showHighlight = true,
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
					},
				},
				cooldown = {
					player = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showInMinutes = true,
						showHighlight = true,				
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
						countdownBothBars = true,
						enableCutoff = false,
						cutoffThreshold = 15000,
					},
					pet = {
						show = true,
						color = {r = 1, g = 0.82, b = 0, a = 1},
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						showInMinutes = true,
						showHighlight = true,			
						highlightColor = {r = 1, g = 1, b = 1, a = 1},
						ignoreIntervalColor = {r = 1, g = 1, b = 1, a = 1},
						countdownBothBars = true,
						enableCutoff = false,
						cutoffThreshold = 15000,
					},
				},
			},
		},
		
-- [OPTIONS FRAME] --

		of = {
			scale = 1,
			border = 10,
			toolbar = {
				icon = {
					length = 22,
					spacing = 12,
				},
				row = {
					spacing = 5,
				},
			},
			panel = {
				header = {
					icon = {
						length = 22
					},
					name = {
						indent = 5,
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 16,
					},
				},
			},
			group = {
				spacing = 15,
				header = {
					icon = {
						length = 22
					},
					name = {
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 16,
					},
				},
				subheader = {
					icon = {
						length = 20
					},
					name = {
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 14,
					},
				},
				cooldown = {
					button = {
						length = 20,
					},
					icon = {
						length = 18,
					},
					name = {
						font = [[Fonts\ZYKai_T.TTF]],
						fontSize = 11,
						indent = 5,
						ignoreIntervalColor = {r = 0.2, g = 0.6, b = 1, a = 1},
						color = {r = 1, g = 1, b = 1, a = 1},
					},
				},
				editBox = {
					width = 40,
				},
			},
		},
-- [MINIMAP FRAME] --
		mf = {
			angle = 90
		},
		zones = {
			-- built below
		},
		ignoreSpells = {
			-- built below
		}
	}	
	local defaultOptions = WOOGLED.defaultOptions;
	-- Set up table structure from zone groups
	for groupName, groupTable in pairs ( Libs.zoneGroups ) do
		if ( groupTable.names[WOOGLED.player.locale] ~= nil ) then
			defaultOptions.zones[groupName] = {};
			defaultOptions.zones[groupName].enableTF = true;
			defaultOptions.zones[groupName].enableFF = true;
		end
	end
	-- Set up table structure from zones
	for zoneName, zoneTable in pairs ( Libs.zones ) do
		if ( zoneTable.groupId == nil or Libs.zoneGroups[zoneTable.groupId].names[WOOGLED.player.locale] == nil ) then
			defaultOptions.zones[zoneName] = {};
			defaultOptions.zones[zoneName].enableTF = true;
			defaultOptions.zones[zoneName].enableFF = true;
		end
	end
	-- Set default zone options
	defaultOptions.zones.other = {enableTF = true, enableFF = true};
	defaultOptions.zones.duel = {enableTF = true, enableFF = true};
end


function WOOGLED.print(pString, pColor)
	if ( pColor ~= nil ) then
		DEFAULT_CHAT_FRAME:AddMessage('\124cff'..pColor..'[WOOGLED] -- '..tostring(pString)..'\124r');
	else
		DEFAULT_CHAT_FRAME:AddMessage('[WOOGLED] -- '..tostring(pString));
	end
end


function WOOGLED.woogsCheck()
	if ( WOOGLED.time.woogsCheck.counter < 5 ) then
		if ( UnitIsFriend('player', 'Woogs-Greymane') or (WOOGLED.player.realm == 'Greymane' and UnitIsFriend('player', 'Woogs') and WOOGLED.player.name ~= 'Woogs') ) then
			WOOGLED.time.woogsCheck.found = WOOGLED.time.woogsCheck.found + 1;
		end
		if ( WOOGLED.time.woogsCheck.found == 2 ) then
			SendChatMessage('WOOOOOGS!', 'BATTLEGROUND', 'Common');
		end
		WOOGLED.time.woogsCheck.counter = WOOGLED.time.woogsCheck.counter + 1;
	else
		WOOGLED.time.woogsCheck.state = false;
		WOOGLED.time.woogsCheck.found = 0;
		WOOGLED.time.woogsCheck.counter = 0;
	end
end


function WOOGLED.setParse(pState)
	if ( pState ) then
		MinimapFrame.frames.anchorBgTexture:SetTexture('Interface\\Icons\\Inv_misc_summerfest_braziergreen');
		WOOGLED.parse = true;
	else
		MinimapFrame.frames.anchorBgTexture:SetTexture('Interface\\Icons\\Inv_misc_summerfest_brazierred');
		WOOGLED.parse = false;
	end
end


function WOOGLED.clearAll()
	Units.deleteAll();
	CooldownFrames.clear();
end


function SlashCmdList.WOOGLED(pMessage)

	if ( pMessage == '' ) then
		OptionsFrame.show();
		
	elseif ( pMessage == 'on' ) then
		WOOGLED.setParse(true);
		WOOGLED.print('Parsing turned on.');
		
	elseif ( pMessage == 'off' ) then
		WOOGLED.setParse(false);
		WOOGLED.print('Parsing turned off.');
		
	elseif ( pMessage == 'clear' ) then
		WOOGLED.clearAll();
		WOOGLED.print('All unit data cleared.');
		
-- [ALL FRAMES] --
	
	elseif ( pMessage == 'default' ) then
		Tools.compareTables('copy', WOOGLED.defaultOptions.tf, WOOGLED_OPTIONS.tf);
		Tools.compareTables('copy', WOOGLED.defaultOptions.ff, WOOGLED_OPTIONS.ff);
		Tools.compareTables('copy', WOOGLED.defaultOptions.ef, WOOGLED_OPTIONS.ef);
		CooldownFrames.setAllOptions('tf');
		CooldownFrames.setAllOptions('ff');
		for i, groupName in pairs ( CooldownFrames.groupsTableIndexByGroupName.ef ) do
			CooldownFrames.setAllOptions(groupName);
		end
		WOOGLED.print('All frame default-options set.');
		
	elseif ( pMessage == 'reset' ) then
		CooldownFrames.reset('tf');
		CooldownFrames.reset('ff');
		CooldownFrames.reset('ef');
		WOOGLED.print('All frames reset.');
		
	elseif ( pMessage == 'show' ) then
		CooldownFrames.setVisibility('tf', false);
		CooldownFrames.setVisibility('ff', false);
		WOOGLED.print('Target Frame and Focus Frame shown.');
		
	elseif ( pMessage == 'hide' ) then
		CooldownFrames.setVisibility('tf', false);
		CooldownFrames.setVisibility('ff', false);
		WOOGLED.print('Target Frame and Focus Frame hidden.');
		
	elseif ( pMessage == 'lock' ) then
		CooldownFrames.setLock('tf', true);
		CooldownFrames.setLock('ff', true);
		CooldownFrames.setLock('ef', true);
		WOOGLED.print('All frames locked.');
		
	elseif ( pMessage == 'unlock' ) then
		CooldownFrames.setLock('tf', false);
		CooldownFrames.setLock('ff', false);
		CooldownFrames.setLock('ef', false);
		WOOGLED.print('All frames unlocked.');
		
	elseif ( pMessage == 'test' ) then
		CooldownFrames.showTestUnit('tf');
		CooldownFrames.showTestUnit('ff');
		for i, groupName in pairs ( CooldownFrames.groupsTableIndexByGroupName.ef ) do
			CooldownFrames.setVisibility(true, groupName);
			CooldownFrames.showTestUnit(groupName);
		end
		WOOGLED.print('All frame tests shown.');
		
-- [TARGET FRAME] --
	
	elseif ( pMessage == 'tf default' ) then
		Tools.compareTables('copy', WOOGLED.defaultOptions.tf, WOOGLED_OPTIONS.tf);
		CooldownFrames.setAllOptions('tf');
		WOOGLED.print('Target Frame default options set.');
		
	elseif ( pMessage == 'tf reset' ) then
		CooldownFrames.reset('tf');
		WOOGLED.print('Target Frame reset.');
		
	elseif ( pMessage == 'tf show' ) then
		CooldownFrames.setVisibility('tf', true);
		WOOGLED.print('Target Frame shown.');
		
	elseif ( pMessage == 'tf hide' ) then
		CooldownFrames.setVisibility('tf', false);
		WOOGLED.print('Target Frame hidden.');
		
	elseif ( pMessage == 'tf lock' ) then
		CooldownFrames.setLock('tf', true);
		WOOGLED.print('Target Frame locked.');
		
	elseif ( pMessage == 'tf unlock' ) then
		CooldownFrames.setLock('tf', false);
		WOOGLED.print('Target Frame unlocked.');
		
	elseif ( pMessage == 'tf test' ) then
		CooldownFrames.showTestUnit('tf');
		WOOGLED.print('Target Frame test shown.');
		
-- [FOCUS FRAME] --
	
	elseif ( pMessage == 'ff default' ) then
		Tools.compareTables('copy', WOOGLED.defaultOptions.ff, WOOGLED_OPTIONS.ff);
		CooldownFrames.setAllOptions('ff');
		WOOGLED.print('Focus Frame default options set.');
		
	elseif ( pMessage == 'ff reset' ) then
		CooldownFrames.reset('ff');
		WOOGLED.print('Focus Frame reset.');
		
	elseif ( pMessage == 'ff show' ) then
		CooldownFrames.setVisibility('ff', true);
		WOOGLED.print('Focus Frame shown.');
		
	elseif ( pMessage == 'ff hide' ) then
		CooldownFrames.setVisibility('ff', false);
		WOOGLED.print('Focus Frame hidden.');
		
	elseif ( pMessage == 'ff lock' ) then
		CooldownFrames.setLock('ff', true);
		WOOGLED.print('Focus Frame locked.');
		
	elseif ( pMessage == 'ff unlock' ) then
		CooldownFrames.setLock('ff', false);
		WOOGLED.print('Focus Frame unlocked.');
		
	elseif ( pMessage == 'ff test' ) then
		CooldownFrames.showTestUnit('ff');
		WOOGLED.print('Focus Frame test shown.');
		
-- [EXTRA FRAMES] --
	
	elseif ( pMessage == 'ef default' ) then
		Tools.compareTables('copy', WOOGLED.defaultOptions.ef, WOOGLED_OPTIONS.ef);
		for i, groupName in pairs ( CooldownFrames.groupsTableIndexByGroupName.ef ) do
			CooldownFrames.setAllOptions(groupName);
		end
		WOOGLED.print('Extra Frames default options set.');
		
	elseif ( pMessage == 'ef reset' ) then
		CooldownFrames.reset('ef');
		WOOGLED.print('Extra Frames reset.');
		
	elseif ( pMessage == 'ef lock' ) then
		CooldownFrames.setLock('ef', true);
		WOOGLED.print('Extra Frames locked.');
		
	elseif ( pMessage == 'ef unlock' ) then
		CooldownFrames.setLock('ef', false);
		WOOGLED.print('Extra Frames unlocked.');
		
	elseif ( pMessage == 'ef test' ) then
		for i, groupName in pairs ( CooldownFrames.groupsTableIndexByGroupName.ef ) do
			CooldownFrames.setVisibility(true, groupName);
			CooldownFrames.showTestUnit(groupName);
		end
		CooldownFrames.showTestUnit('ef');
		WOOGLED.print('Extra Frames test shown.');
		
-- [SAVE GLOBAL NAMES] --
		
	elseif ( pMessage == 'save globals' ) then
		WOOGLED_OPTIONS.globals = {};
		for k, v in pairs (_G) do
			if ( string.find(k, '^WOOGLED.*') ) then
				table.insert(WOOGLED_OPTIONS.globals, k);
			end
		end
		table.sort(WOOGLED_OPTIONS.globals);
		WOOGLED.print('Global names saved.');
		
-- [DYNAMIC COMMANDS] --

	else
		local spacePosition = strfind(pMessage, ' ');
		local messagePrefix = strsub(pMessage, 1, spacePosition);
		if ( messagePrefix == 'build ' ) then
			local unitId = strsub(pMessage, 7);
			if ( CooldownFrames.buildUnitIdFrame(unitId) ) then
				WOOGLED.print("Extra Frame created for '"..unitId.."'.");
			end

-- [UNKNOWN COMMAND] --
			
		else
			WOOGLED.print('------');
			WOOGLED.print('Show Options Frame:   /woogled');
			WOOGLED.print('Parsing:    /woogled [on/off]');
			WOOGLED.print('Clear Data:    /woogled clear');
			WOOGLED.print('---');
			WOOGLED.print('Target Frame:  /woogled tf [default/reset/show/hide/lock/unlock]');
			WOOGLED.print('Focus Frame:   /woogled ff [default/reset/show/hide/lock/unlock]');
			WOOGLED.print('Extra Frames:   /woogled ef [default/reset/lock/unlock]');
			WOOGLED.print('All Frames:   /woogled [default/reset/show/hide/lock/unlock]');
			WOOGLED.print('------');
		end
	end
end